@@ -80,6 +80,7 @@ class GenericSource(object):
 	@property
 	def identHash(self):
 		if not self.__identHash:
+			# Calculate the ident hash
 			h = hashlib.new(self.IDENT_HASH, self.SRCTYPE.encode("utf-8"))
 			if self.name is not None:
 				h.update(self.name.encode("utf-8"))
@@ -89,6 +90,11 @@ class GenericSource(object):
 			self.__identHash = h.digest()
 		return self.__identHash
 
+	@identHash.setter
+	def identHash(self, identHash):
+		# Force the ident hash.
+		self.__identHash = identHash
+
 	@property
 	def identHashStr(self):
 		return binascii.b2a_hex(self.identHash).decode("ascii")
@@ -187,16 +193,85 @@ class GuiSettings(object):
 	def getEditorFont(self):
 		return self.editorFont
 
+class CoreLinkSettings(object):
+	def __init__(self,
+		     spawnLocalEn=True,
+		     spawnLocalPortRange=range(4151, 4151 + 4095 + 1),
+		     spawnLocalInterpreters="pypy3; pypy; $CURRENT; "\
+					    "python3; python2; python; py",
+		     connectHost="localhost",
+		     connectPort=4151,
+		     connectTimeoutMs=3000):
+		self.setSpawnLocalEn(spawnLocalEn)
+		self.setSpawnLocalPortRange(spawnLocalPortRange)
+		self.setSpawnLocalInterpreters(spawnLocalInterpreters),
+		self.setConnectHost(connectHost)
+		self.setConnectPort(connectPort)
+		self.setConnectTimeoutMs(connectTimeoutMs)
+
+	def setSpawnLocalEn(self, spawnLocalEn):
+		self.spawnLocalEn = spawnLocalEn
+
+	def getSpawnLocalEn(self):
+		return self.spawnLocalEn
+
+	def setSpawnLocalPortRange(self, spawnLocalPortRange):
+		self.spawnLocalPortRange = spawnLocalPortRange
+
+	def getSpawnLocalPortRange(self):
+		return self.spawnLocalPortRange
+
+	def setSpawnLocalInterpreters(self, spawnLocalInterpreters):
+		self.spawnLocalInterpreters = spawnLocalInterpreters
+
+	def getSpawnLocalInterpreters(self):
+		return self.spawnLocalInterpreters
+
+	def getSpawnLocalInterpreterList(self, replace=True):
+		ret = []
+		for interp in self.getSpawnLocalInterpreters().split(';'):
+			interp = interp.strip()
+			if replace:
+				if interp == "$CURRENT":
+					interp = sys.executable
+			ret.append(interp)
+		return ret
+
+	def setConnectHost(self, connectHost):
+		self.connectHost = connectHost
+
+	def getConnectHost(self):
+		return self.connectHost
+
+	def setConnectPort(self, connectPort):
+		self.connectPort = connectPort
+
+	def getConnectPort(self):
+		return self.connectPort
+
+	def setConnectTimeoutMs(self, connectTimeoutMs):
+		self.connectTimeoutMs = connectTimeoutMs
+
+	def getConnectTimeoutMs(self):
+		return self.connectTimeoutMs
+
 class Project(object):
+	DATETIME_FMT	= "%Y-%m-%d %H:%M:%S.%f"
+
 	def __init__(self, projectFile,
+		     createDate=None,
+		     modifyDate=None,
 		     awlSources=[],
 		     symTabSources=[],
 		     libSelections=[],
 		     cpuSpecs=None,
 		     obTempPresetsEn=False,
 		     extInsnsEn=False,
-		     guiSettings=None):
+		     guiSettings=None,
+		     coreLinkSettings=None):
 		self.setProjectFile(projectFile)
+		self.setCreateDate(createDate)
+		self.setModifyDate(modifyDate)
 		self.setAwlSources(awlSources)
 		self.setSymTabSources(symTabSources)
 		self.setLibSelections(libSelections)
@@ -208,6 +283,9 @@ class Project(object):
 		if not guiSettings:
 			guiSettings = GuiSettings()
 		self.setGuiSettings(guiSettings)
+		if not coreLinkSettings:
+			coreLinkSettings = CoreLinkSettings()
+		self.setCoreLinkSettings(coreLinkSettings)
 
 	def setProjectFile(self, filename):
 		self.projectFile = filename
@@ -215,6 +293,24 @@ class Project(object):
 	def getProjectFile(self):
 		return self.projectFile
 
+	def setCreateDate(self, createDate):
+		if createDate is None:
+			createDate = datetime.datetime.utcnow()
+		self.createDate = createDate
+
+	def getCreateDate(self):
+		return self.createDate
+
+	def setModifyDate(self, modifyDate):
+		if modifyDate is None:
+			modifyDate = self.getCreateDate()
+		if modifyDate is None:
+			modifyDate = datetime.datetime.utcnow()
+		self.modifyDate = modifyDate
+
+	def getModifyDate(self):
+		return self.modifyDate
+
 	def setAwlSources(self, awlSources):
 		self.awlSources = awlSources
 
@@ -257,6 +353,12 @@ class Project(object):
 	def getGuiSettings(self):
 		return self.guiSettings
 
+	def setCoreLinkSettings(self, coreLinkSettings):
+		self.coreLinkSettings = coreLinkSettings
+
+	def getCoreLinkSettings(self):
+		return self.coreLinkSettings
+
 	@classmethod
 	def dataIsProject(cls, data):
 		magic = b"[AWLSIM_PROJECT]"
@@ -279,6 +381,8 @@ class Project(object):
 			raise AwlSimError("Project file: The data is "\
 				"not an awlsim project.")
 		projectDir = os.path.dirname(projectFile)
+		createDate = None
+		modifyDate = None
 		awlSources = []
 		symTabSources = []
 		libSelections = []
@@ -286,15 +390,32 @@ class Project(object):
 		obTempPresetsEn = False
 		extInsnsEn = False
 		guiSettings = GuiSettings()
+		linkSettings = CoreLinkSettings()
 		try:
 			p = _ConfigParser()
 			p.readfp(StringIO(text), projectFile)
+
+			# AWLSIM_PROJECT section
 			version = p.getint("AWLSIM_PROJECT", "file_version")
 			expectedVersion = 0
 			if version != expectedVersion:
 				raise AwlSimError("Project file: Unsupported version. "\
 					"File version is %d, but expected %d." %\
 					(version, expectedVersion))
+			if p.has_option("AWLSIM_PROJECT", "date"):
+				# Compatibility only. "date" is deprecated.
+				dStr = p.get("AWLSIM_PROJECT", "date")
+				createDate = datetime.datetime.strptime(dStr,
+							cls.DATETIME_FMT)
+				modifyDate = createDate
+			if p.has_option("AWLSIM_PROJECT", "create_date"):
+				dStr = p.get("AWLSIM_PROJECT", "create_date")
+				createDate = datetime.datetime.strptime(dStr,
+							cls.DATETIME_FMT)
+			if p.has_option("AWLSIM_PROJECT", "modify_date"):
+				dStr = p.get("AWLSIM_PROJECT", "modify_date")
+				modifyDate = datetime.datetime.strptime(dStr,
+							cls.DATETIME_FMT)
 
 			# CPU section
 			for i in range(0xFFFF):
@@ -399,6 +520,47 @@ class Project(object):
 							     effectiveEntryIndex = effBlock)
 				)
 
+			# CORE_LINK section
+			if p.has_option("CORE_LINK", "spawn_local"):
+				linkSettings.setSpawnLocalEn(
+					p.getboolean("CORE_LINK", "spawn_local"))
+			if p.has_option("CORE_LINK", "spawn_local_port_range"):
+				pRange = p.get("CORE_LINK", "spawn_local_port_range")
+				try:
+					pRange = pRange.split(":")
+					begin = int(pRange[0])
+					end = int(pRange[1])
+					if end < begin:
+						raise ValueError
+					pRange = range(begin, end + 1)
+				except (IndexError, ValueError) as e:
+					raise AwlSimError("Project file: Invalid port range")
+				linkSettings.setSpawnLocalPortRange(pRange)
+			if p.has_option("CORE_LINK", "spawn_local_interpreters"):
+				interp = p.get("CORE_LINK", "spawn_local_interpreters")
+				try:
+					interp = base64.b64decode(interp)
+					interp = interp.decode("utf-8")
+				except (TypeError, binascii.Error, UnicodeError) as e:
+					raise AwlSimError("Project file: "
+						"Invalid interpreter list")
+				linkSettings.setSpawnLocalInterpreters(interp)
+			if p.has_option("CORE_LINK", "connect_host"):
+				host = p.get("CORE_LINK", "connect_host")
+				try:
+					host = base64.b64decode(host)
+					host = host.decode("utf-8")
+				except (TypeError, binascii.Error, UnicodeError) as e:
+					raise AwlSimError("Project file: "
+						"Invalid host name")
+				linkSettings.setConnectHost(host)
+			if p.has_option("CORE_LINK", "connect_port"):
+				port = p.getint("CORE_LINK", "connect_port")
+				linkSettings.setConnectPort(port)
+			if p.has_option("CORE_LINK", "connect_timeout_ms"):
+				timeout = p.getint("CORE_LINK", "connect_timeout_ms")
+				linkSettings.setConnectTimeoutMs(timeout)
+
 			# GUI section
 			if p.has_option("GUI", "editor_autoindent"):
 				guiSettings.setEditorAutoIndentEn(
@@ -416,13 +578,16 @@ class Project(object):
 			raise AwlSimError("Project parser error: " + str(e))
 
 		return cls(projectFile = projectFile,
+			   createDate = createDate,
+			   modifyDate = modifyDate,
 			   awlSources = awlSources,
 			   symTabSources = symTabSources,
 			   libSelections = libSelections,
 			   cpuSpecs = cpuSpecs,
 			   obTempPresetsEn = obTempPresetsEn,
 			   extInsnsEn = extInsnsEn,
-			   guiSettings = guiSettings)
+			   guiSettings = guiSettings,
+			   coreLinkSettings = linkSettings)
 
 	@classmethod
 	def fromFile(cls, filename):
@@ -455,10 +620,15 @@ class Project(object):
 			projectFile = self.projectFile
 		projectDir = os.path.dirname(projectFile)
 
+		self.setModifyDate(datetime.datetime.utcnow())
+
 		lines = []
 		lines.append("[AWLSIM_PROJECT]")
 		lines.append("file_version=0")
-		lines.append("date=%s" % str(datetime.datetime.utcnow()))
+		lines.append("create_date=%s" %\
+			     self.getCreateDate().strftime(self.DATETIME_FMT))
+		lines.append("modify_date=%s" %\
+			     self.getModifyDate().strftime(self.DATETIME_FMT))
 		lines.append("")
 
 		lines.append("[CPU]")
@@ -504,14 +674,36 @@ class Project(object):
 				i, libSel.getEffectiveEntryIndex()))
 		lines.append("")
 
+		lines.append("[CORE_LINK]")
+		linkSettings = self.getCoreLinkSettings()
+		lines.append("spawn_local=%d" %\
+			     int(bool(linkSettings.getSpawnLocalEn())))
+		lines.append("spawn_local_port_range=%d:%d" %(\
+			     linkSettings.getSpawnLocalPortRange()[0],
+			     linkSettings.getSpawnLocalPortRange()[-1]))
+		interp = linkSettings.getSpawnLocalInterpreters()
+		interp = interp.encode("utf-8", "ignore")
+		interp = base64.b64encode(interp).decode("ascii")
+		lines.append("spawn_local_interpreters=%s" % interp)
+		host = linkSettings.getConnectHost()
+		host = host.encode("utf-8", "ignore")
+		host = base64.b64encode(host).decode("ascii")
+		lines.append("connect_host=%s" % host)
+		lines.append("connect_port=%d" %\
+			     int(linkSettings.getConnectPort()))
+		lines.append("connect_timeout_ms=%d" %\
+			     int(linkSettings.getConnectTimeoutMs()))
+		lines.append("")
+
 		lines.append("[GUI]")
+		guiSettings = self.getGuiSettings()
 		lines.append("editor_autoindent=%d" %\
-			     int(bool(self.getGuiSettings().getEditorAutoIndentEn())))
+			     int(bool(guiSettings.getEditorAutoIndentEn())))
 		lines.append("editor_paste_autoindent=%d" %\
-			     int(bool(self.getGuiSettings().getEditorPasteIndentEn())))
+			     int(bool(guiSettings.getEditorPasteIndentEn())))
 		lines.append("editor_validation=%d" %\
-			     int(bool(self.getGuiSettings().getEditorValidationEn())))
-		lines.append("editor_font=%s" % self.getGuiSettings().getEditorFont())
+			     int(bool(guiSettings.getEditorValidationEn())))
+		lines.append("editor_font=%s" % guiSettings.getEditorFont())
 		lines.append("")
 
 		return "\r\n".join(lines)
