@@ -1,5 +1,5 @@
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
 
 from awlsim.core.instructions.insn_abs import * #@nocy
 from awlsim.core.instructions.insn_acos import * #@nocy
