@@ -19,20 +19,20 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
 
 import math
 import re
 
 #from awlsim.core.statusword cimport * #@cy
 
+from awlsim.common.cpuspecs import *
 from awlsim.core.util import *
 from awlsim.core.operators import *
 from awlsim.core.parameters import *
 from awlsim.core.datatypes import *
 from awlsim.core.parser import *
 from awlsim.core.timers import *
-from awlsim.core.cpuspecs import *
 
 
 class OpDescriptor(object):
