@@ -38,7 +38,9 @@ import time
 class AwlSimClient(object):
 	def __init__(self):
 		self.serverProcess = None
-		self.transceiver = None
+		self.serverProcessHost = None
+		self.serverProcessPort = None
+		self.__transceiver = None
 
 	def spawnServer(self,
 			interpreter=None,
@@ -88,19 +90,46 @@ class AwlSimClient(object):
 					"any of the supplied Python interpreters: %s\n"
 					"No interpreter found." %\
 					str(toList(interpreter)))
+		self.serverProcessHost = listenHost
+		self.serverProcessPort = listenPort
 		if isJython:
 			#XXX Workaround: Jython's socket module does not like connecting
 			# to a starting server. Wait a few seconds for the server
 			# to start listening on the socket.
 			time.sleep(10)
 
+	def killSpawnedServer(self):
+		"""Shutdown the server process started with spawnServer()."""
+
+		if not self.serverProcess:
+			return
+
+		if self.__transceiver:
+			try:
+				msg = AwlSimMessage_SHUTDOWN()
+				status = self.__sendAndWaitFor_REPLY(msg)
+				if status != AwlSimMessage_REPLY.STAT_OK:
+					printError("AwlSimClient: Failed to shut "
+						"down server via message")
+			except (AwlSimError, MaintenanceRequest) as e:
+				pass
+
+		self.serverProcess.terminate()
+		self.serverProcess.wait()
+		self.serverProcess = None
+		self.serverProcessHost = None
+		self.serverProcessPort = None
+
 	def connectToServer(self,
 			    host=AwlSimServer.DEFAULT_HOST,
-			    port=AwlSimServer.DEFAULT_PORT):
+			    port=AwlSimServer.DEFAULT_PORT,
+			    timeout=3.0):
 		"""Connect to a AwlSim-core server.
 		host -> The hostname or IP address to connect to.
 		port -> The port to connect to."""
 
+		startTime = monotonic_time()
+		readableSockaddr = host
 		try:
 			family, socktype, sockaddr = AwlSimServer.getaddrinfo(host, port)
 			if family == AF_UNIX:
@@ -109,17 +138,15 @@ class AwlSimClient(object):
 				readableSockaddr = "%s:%d" % (sockaddr[0], sockaddr[1])
 			printInfo("AwlSimClient: Connecting to server '%s'..." % readableSockaddr)
 			sock = socket.socket(family, socktype)
-			count = 0
 			while 1:
+				if monotonic_time() - startTime > timeout:
+					raise AwlSimError("Timeout connecting "
+						"to AwlSimServer %s" % readableSockaddr)
 				try:
 					sock.connect(sockaddr)
-				except (OSError, socket.error) as e:
+				except SocketErrors as e:
 					if e.errno == errno.ECONNREFUSED or\
 					   e.errno == errno.ENOENT:
-						count += 1
-						if count >= 100:
-							raise AwlSimError("Timeout connecting "
-								"to AwlSimServer %s" % readableSockaddr)
 						self.sleep(0.1)
 						continue
 					if isJython and\
@@ -129,17 +156,17 @@ class AwlSimClient(object):
 						continue
 					raise
 				break
-		except socket.error as e:
+		except SocketErrors as e:
 			raise AwlSimError("Failed to connect to AwlSimServer %s: %s" %\
 				(readableSockaddr, str(e)))
 		printInfo("AwlSimClient: Connected.")
-		self.transceiver = AwlSimMessageTransceiver(sock, readableSockaddr)
+		self.__transceiver = AwlSimMessageTransceiver(sock, readableSockaddr)
 		self.lastRxMsg = None
 
 		# Ping the server
 		try:
-			self.transceiver.send(AwlSimMessage_PING())
-			msg = self.transceiver.receive(timeout = 5.0)
+			self.__transceiver.send(AwlSimMessage_PING())
+			msg = self.__transceiver.receive(timeout = 2.0)
 			if not msg:
 				raise AwlSimError("AwlSimClient: Server did not "
 					"respond to PING request.")
@@ -151,26 +178,20 @@ class AwlSimClient(object):
 		except TransferError as e:
 			raise AwlSimError("AwlSimClient: PING to server failed")
 
+	def shutdownTransceiver(self):
+		"""Shutdown transceiver and close the socket."""
+
+		if not self.__transceiver:
+			return
+
+		self.__transceiver.shutdown()
+		self.__transceiver = None
+
 	def shutdown(self):
 		"""Shutdown all sockets and spawned processes."""
 
-		if self.serverProcess:
-			if self.transceiver:
-				try:
-					msg = AwlSimMessage_SHUTDOWN()
-					status = self.__sendAndWaitFor_REPLY(msg)
-					if status != AwlSimMessage_REPLY.STAT_OK:
-						printError("AwlSimClient: Failed to shut "
-							"down server via message")
-				except (AwlSimError, MaintenanceRequest) as e:
-					pass
-
-			self.serverProcess.terminate()
-			self.serverProcess.wait()
-			self.serverProcess = None
-		if self.transceiver:
-			self.transceiver.shutdown()
-			self.transceiver = None
+		self.killSpawnedServer()
+		self.shutdownTransceiver()
 
 	def __rx_NOP(self, msg):
 		pass # Nothing
@@ -179,7 +200,7 @@ class AwlSimClient(object):
 		raise msg.exception
 
 	def __rx_PING(self, msg):
-		self.transceiver.send(AwlSimMessage_PONG())
+		self.__send(AwlSimMessage_PONG())
 
 	def handle_PONG(self):
 		printInfo("AwlSimClient: Received PONG")
@@ -212,6 +233,12 @@ class AwlSimClient(object):
 	def __rx_INSNSTATE(self, msg):
 		self.handle_INSNSTATE(msg)
 
+	def handle_IDENTS(self, msg):
+		pass # Don't do anything by default
+
+	def __rx_IDENTS(self, msg):
+		self.handle_IDENTS(msg)
+
 	__msgRxHandlers = {
 		AwlSimMessage.MSG_ID_REPLY		: __rx_NOP,
 		AwlSimMessage.MSG_ID_EXCEPTION		: __rx_EXCEPTION,
@@ -222,6 +249,8 @@ class AwlSimClient(object):
 		AwlSimMessage.MSG_ID_CPUSPECS		: __rx_NOP,
 		AwlSimMessage.MSG_ID_MEMORY		: __rx_MEMORY,
 		AwlSimMessage.MSG_ID_INSNSTATE		: __rx_INSNSTATE,
+		AwlSimMessage.MSG_ID_RUNSTATE		: __rx_NOP,
+		AwlSimMessage.MSG_ID_IDENTS		: __rx_IDENTS,
 	}
 
 	# Main message processing
@@ -230,25 +259,22 @@ class AwlSimClient(object):
 	#          x -> Timeout, in seconds.
 	def processMessages(self, timeout=None):
 		self.lastRxMsg = None
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		try:
-			msg = self.transceiver.receive(timeout)
-		except (socket.error, BlockingIOError) as e:
-			if isinstance(e, socket.timeout) or\
-			   isinstance(e, BlockingIOError) or\
-			   e.errno == errno.EAGAIN or\
-			   e.errno == errno.EWOULDBLOCK:
+			msg = self.__transceiver.receive(timeout)
+		except TransferError as e:
+			if e.reason == e.REASON_BLOCKING:
 				return False
 			raise AwlSimError("AwlSimClient: "
 				"I/O error in connection to server '%s':\n"
 				"%s (errno = %s)" %\
-				(self.transceiver.peerInfoString, str(e), str(e.errno)))
-		except (AwlSimMessageTransceiver.RemoteEndDied, TransferError) as e:
+				(self.__transceiver.peerInfoString, str(e), str(e.errno)))
+		except TransferError as e:
 			raise AwlSimError("AwlSimClient: "
 				"Connection to server '%s' died. "
 				"Failed to receive message." %\
-				self.transceiver.peerInfoString)
+				self.__transceiver.peerInfoString)
 		if not msg:
 			return False
 		self.lastRxMsg = msg
@@ -263,46 +289,79 @@ class AwlSimClient(object):
 	def sleep(self, seconds):
 		time.sleep(seconds)
 
-	def __sendAndWait(self, txMsg, checkRxMsg, waitTimeout=3.0):
-		self.transceiver.send(txMsg)
+	def __send(self, txMsg):
+		try:
+			self.__transceiver.send(txMsg)
+		except TransferError as e:
+			raise AwlSimError("AwlSimClient: "
+				"Send error in connection to server '%s':\n"
+				"%s (errno = %s)" %\
+				(self.__transceiver.peerInfoString,
+				 str(e), str(e.errno)))
+
+	def __sendAndWait(self, txMsg, checkRxMsg,
+			  waitTimeout=3.0,
+			  ignoreMaintenanceRequests=False):
+		self.__send(txMsg)
 		now = monotonic_time()
 		end = now + waitTimeout
 		while now < end:
-			if self.processMessages(0.1):
-				rxMsg = self.lastRxMsg
-				if checkRxMsg(rxMsg):
-					return rxMsg
+			try:
+				if self.processMessages(0.1):
+					rxMsg = self.lastRxMsg
+					if checkRxMsg(rxMsg):
+						return rxMsg
+			except MaintenanceRequest as e:
+				if not ignoreMaintenanceRequests:
+					raise e
 			now = monotonic_time()
 		raise AwlSimError("AwlSimClient: Timeout waiting for server reply.")
 
-	def __sendAndWaitFor_REPLY(self, msg, timeout=3.0):
+	def __sendAndWaitFor_REPLY(self, msg, timeout=3.0,
+				   ignoreMaintenanceRequests=False):
 		def checkRxMsg(rxMsg):
 			return rxMsg.msgId == AwlSimMessage.MSG_ID_REPLY and\
 			       rxMsg.inReplyToId == msg.msgId and\
 			       rxMsg.inReplyToSeq == msg.seq
-		return self.__sendAndWait(msg, checkRxMsg, timeout).status
+		return self.__sendAndWait(msg, checkRxMsg, timeout,
+					  ignoreMaintenanceRequests).status
 
 	def reset(self):
+		if not self.__transceiver:
+			return False
 		msg = AwlSimMessage_RESET()
-		status = self.__sendAndWaitFor_REPLY(msg)
+		status = self.__sendAndWaitFor_REPLY(msg,
+			ignoreMaintenanceRequests = True)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to reset CPU")
+		return True
 
 	def setRunState(self, run=True):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		if run:
 			runState = AwlSimMessage_RUNSTATE.STATE_RUN
 		else:
 			runState = AwlSimMessage_RUNSTATE.STATE_STOP
 		msg = AwlSimMessage_RUNSTATE(runState)
-		status = self.__sendAndWaitFor_REPLY(msg)
+		status = self.__sendAndWaitFor_REPLY(msg,
+			ignoreMaintenanceRequests = True)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to set run state")
 		return True
 
+	def getRunState(self):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_GET_RUNSTATE()
+		rxMsg = self.__sendAndWait(msg,
+			lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_RUNSTATE)
+		if rxMsg.runState == AwlSimMessage_RUNSTATE.STATE_RUN:
+			return True
+		return False
+
 	def loadCode(self, codeSource):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		msg = AwlSimMessage_LOAD_CODE(codeSource)
 		status = self.__sendAndWaitFor_REPLY(msg, 10.0)
@@ -311,19 +370,25 @@ class AwlSimClient(object):
 		return True
 
 	def loadSymbolTable(self, symTabSource):
+		if not self.__transceiver:
+			return False
 		msg = AwlSimMessage_LOAD_SYMTAB(symTabSource)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load symbol table")
+		return True
 
 	def loadLibraryBlock(self, libSelection):
+		if not self.__transceiver:
+			return False
 		msg = AwlSimMessage_LOAD_LIB(libSelection)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load library block")
+		return True
 
 	def loadHardwareModule(self, name, parameters={}):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		msg = AwlSimMessage_LOAD_HW(name = name,
 					    paramDict = parameters)
@@ -332,8 +397,24 @@ class AwlSimClient(object):
 			raise AwlSimError("AwlSimClient: Failed to load hardware module")
 		return True
 
+	# Request the (source) ident hashes from the CPU.
+	# This method is asynchronous.
+	# The idents are returned via handle_IDENTS()
+	def requestIdents(self, reqAwlSources = False,
+			  reqSymTabSources = False,
+			  reqHwModules = False,
+			  reqLibSelections = False):
+		if not self.__transceiver:
+			return False
+		self.__send(AwlSimMessage_GET_IDENTS(
+			(AwlSimMessage_GET_IDENTS.GET_AWLSRCS if reqAwlSources else 0) |\
+			(AwlSimMessage_GET_IDENTS.GET_SYMTABSRCS if reqSymTabSources else 0) |\
+			(AwlSimMessage_GET_IDENTS.GET_HWMODS if reqHwModules else 0) |\
+			(AwlSimMessage_GET_IDENTS.GET_LIBSELS if reqLibSelections else 0)))
+		return True
+
 	def __setOption(self, name, value, sync=True):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		msg = AwlSimMessage_SET_OPT(name, str(value))
 		if sync:
@@ -341,7 +422,7 @@ class AwlSimClient(object):
 			if status != AwlSimMessage_REPLY.STAT_OK:
 				raise AwlSimError("AwlSimClient: Failed to set option '%s'" % name)
 		else:
-			self.transceiver.send(msg)
+			self.__send(msg)
 		return True
 
 	def setLoglevel(self, level=Logging.LOG_INFO):
@@ -369,7 +450,7 @@ class AwlSimClient(object):
 	def setInsnStateDump(self, enable=True,
 			     sourceId=0, fromLine=1, toLine=0x7FFFFFFF,
 			     sync=True):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return None
 		msg = AwlSimMessage_INSNSTATE_CONFIG(
 			flags = 0,
@@ -386,10 +467,10 @@ class AwlSimClient(object):
 			if status != AwlSimMessage_REPLY.STAT_OK:
 				raise AwlSimError("AwlSimClient: Failed to set insn state dump")
 		else:
-			self.transceiver.send(msg)
+			self.__send(msg)
 
 	def getCpuSpecs(self):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return None
 		msg = AwlSimMessage_GET_CPUSPECS()
 		rxMsg = self.__sendAndWait(msg,
@@ -397,7 +478,7 @@ class AwlSimClient(object):
 		return rxMsg.cpuspecs
 
 	def setCpuSpecs(self, cpuspecs):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		msg = AwlSimMessage_CPUSPECS(cpuspecs)
 		status = self.__sendAndWaitFor_REPLY(msg)
@@ -413,7 +494,7 @@ class AwlSimClient(object):
 	#  - repeat on every n'th cycle (repetitionFactor=n)
 	# If sync is true, wait for a reply from the server.
 	def setMemoryReadRequests(self, memAreas, repetitionFactor=0, sync=False):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		msg = AwlSimMessage_REQ_MEMORY(0, repetitionFactor, memAreas)
 		if sync:
@@ -423,14 +504,14 @@ class AwlSimClient(object):
 				raise AwlSimError("AwlSimClient: Failed to set "
 					"memory read reqs")
 		else:
-			self.transceiver.send(msg)
+			self.__send(msg)
 		return True
 
 	# Write memory areas in the server.
 	# memAreas is a list of MemoryAreaData instances.
 	# If sync is true, wait for a reply from the server.
 	def writeMemory(self, memAreas, sync=False):
-		if not self.transceiver:
+		if not self.__transceiver:
 			return False
 		msg = AwlSimMessage_MEMORY(0, memAreas)
 		if sync:
@@ -440,5 +521,5 @@ class AwlSimClient(object):
 				raise AwlSimError("AwlSimClient: Failed to write "
 					"to memory")
 		else:
-			self.transceiver.send(msg)
+			self.__send(msg)
 		return True
