@@ -33,8 +33,41 @@ import socket
 import errno
 
 
+SocketErrors = (socket.error, BlockingIOError, ConnectionError)
+
 class TransferError(Exception):
-	pass
+	EnumGen.start
+	REASON_UNKNOWN		= EnumGen.item
+	REASON_BLOCKING		= EnumGen.item
+	REASON_REMOTEDIED	= EnumGen.item
+	EnumGen.end
+
+	def __init__(self, text, parentException=None, reason=None):
+		if not text and parentException:
+			text = str(parentException)
+		if hasattr(parentException, "errno"):
+			_errno = parentException.errno
+		else:
+			_errno = errno.ECONNREFUSED
+		if reason is None:
+			if parentException:
+				# Try to find out whether this was an exception due
+				# to blocking IO (on a nonblocking socket).
+				# This varies between Python versions, argh.
+				if isinstance(parentException, socket.timeout) or\
+				   isinstance(parentException, BlockingIOError) or\
+				   _errno == errno.EAGAIN or\
+				   _errno == errno.EWOULDBLOCK or\
+				   _errno == errno.EINTR:
+					reason = self.REASON_BLOCKING
+				else:
+					reason = self.REASON_UNKNOWN
+			else:
+				reason = self.REASON_UNKNOWN
+		Exception.__init__(self, text)
+		self.parent = parentException
+		self.reason = reason
+		self.errno = _errno
 
 class AwlSimMessage(object):
 	# Header format:
@@ -70,6 +103,9 @@ class AwlSimMessage(object):
 	MSG_ID_INSNSTATE	= EnumGen.item
 	MSG_ID_INSNSTATE_CONFIG	= EnumGen.item
 	MSG_ID_LOAD_LIB		= EnumGen.item
+	MSG_ID_GET_RUNSTATE	= EnumGen.item
+	MSG_ID_GET_IDENTS	= EnumGen.item
+	MSG_ID_IDENTS		= EnumGen.item
 	EnumGen.end
 
 	_bytesLenStruct = struct.Struct(str(">I"))
@@ -202,6 +238,10 @@ class AwlSimMessage_RUNSTATE(AwlSimMessage):
 			raise TransferError("RUNSTATE: Invalid data format")
 		return cls(runState)
 
+class AwlSimMessage_GET_RUNSTATE(AwlSimMessage):
+	def __init__(self):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_GET_RUNSTATE)
+
 class AwlSimMessage_EXCEPTION(AwlSimMessage):
 	def __init__(self, exception):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_EXCEPTION)
@@ -306,33 +346,44 @@ class AwlSimMessage_LOAD_LIB(AwlSimMessage):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_LOAD_LIB)
 		self.libSelection = libSelection
 
+	# Pack a library selection.
+	# May raise ValueError or struct.error
+	@classmethod
+	def packLibSelection(cls, libSel):
+		payload = [ cls.packString(libSel.getLibName()), ]
+		payload.append(cls.plStruct.pack(libSel.getEntryType(),
+						 libSel.getEntryIndex(),
+						 libSel.getEffectiveEntryIndex()))
+		return b''.join(payload)
+
+	# Unpack a library selection.
+	# May raise ValueError, struct.error or AwlSimError
+	@classmethod
+	def unpackLibSelection(cls, payload, offset = 0):
+		libName, count = cls.unpackString(payload, offset)
+		offset += count
+		eType, eIndex, effIndex =\
+			cls.plStruct.unpack_from(payload, offset)
+		offset += cls.plStruct.size
+		return (AwlLibEntrySelection(
+				libName = libName,
+				entryType = eType,
+				entryIndex = eIndex,
+				effectiveEntryIndex = effIndex),
+			offset)
+
 	def toBytes(self):
-		payload = b""
 		try:
-			payload += self.packString(self.libSelection.getLibName())
-			payload += self.plStruct.pack(self.libSelection.getEntryType(),
-						      self.libSelection.getEntryIndex(),
-						      self.libSelection.getEffectiveEntryIndex())
+			payload = self.packLibSelection(self.libSelection)
 			return AwlSimMessage.toBytes(self, len(payload)) + payload
-		except (ValueError) as e:
+		except (ValueError, struct.error) as e:
 			raise TransferError("LOAD_LIB: Invalid data format")
 
 	@classmethod
 	def fromBytes(cls, payload):
-		libSelection = AwlLibEntrySelection()
-		offset = 0
 		try:
-			libName, count = cls.unpackString(payload, offset)
-			offset += count
-			eType, eIndex, effIndex =\
-				cls.plStruct.unpack_from(payload, offset)
-			offset += cls.plStruct.size
-			libSelection = AwlLibEntrySelection(
-				libName = libName,
-				entryType = eType,
-				entryIndex = eIndex,
-				effectiveEntryIndex = effIndex)
-		except (ValueError, AwlSimError) as e:
+			libSelection, offset = cls.unpackLibSelection(payload)
+		except (ValueError, struct.error, AwlSimError) as e:
 			raise TransferError("LOAD_LIB: Invalid data format")
 		return cls(libSelection = libSelection)
 
@@ -554,7 +605,7 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 							 memArea.start,
 							 memArea.length,
 							 actualLength))
-			pl.append(memArea.data)
+			pl.append(bytes(memArea.data))
 			# Pad to a 32-bit boundary
 			pl.append(b'\x00' * (roundUp(actualLength, 4) - actualLength))
 		pl = b''.join(pl)
@@ -669,9 +720,144 @@ class AwlSimMessage_INSNSTATE_CONFIG(AwlSimMessage):
 			raise TransferError("INSNSTATE_CONFIG: Invalid data format")
 		return cls(flags, sourceId, fromLine, toLine)
 
-class AwlSimMessageTransceiver(object):
-	class RemoteEndDied(Exception): pass
+class AwlSimMessage_GET_IDENTS(AwlSimMessage):
+	# Get-flags. Specify what information to get.
+	EnumGen.start
+	GET_AWLSRCS		= EnumGen.bitmask # Get AwlSource()s (w/o data)
+	GET_SYMTABSRCS		= EnumGen.bitmask # Get SymTabSource()s (w/o data)
+	GET_HWMODS		= EnumGen.bitmask # Get HW modules
+	GET_LIBSELS		= EnumGen.bitmask # Get AwlLibEntrySelection()s
+	EnumGen.end
 
+	# Payload header struct:
+	#	Get-flags (32 bit)
+	#	Reserved (32 bit)
+	plHdrStruct = struct.Struct(str(">II"))
+
+	def __init__(self, getFlags):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_GET_IDENTS)
+		self.getFlags = getFlags
+
+	def toBytes(self):
+		payload = self.plHdrStruct.pack(self.getFlags, 0)
+		return AwlSimMessage.toBytes(self, len(payload)) + payload
+
+	@classmethod
+	def fromBytes(cls, payload):
+		try:
+			getFlags, _unused = cls.plHdrStruct.unpack_from(payload, 0)
+		except (ValueError, struct.error) as e:
+			raise TransferError("GET_IDENTS: Invalid data format")
+		return cls(getFlags)
+
+class AwlSimMessage_IDENTS(AwlSimMessage):
+	# Payload header struct:
+	#	Number of AWL sources (32 bit)
+	#	Number of symbol tables (32 bit)
+	#	Number of hardware modules (32 bit)
+	#	Number of library selections (32 bit)
+	#	Reserved (32 bit)
+	#	Reserved (32 bit)
+	plHdrStruct = struct.Struct(str(">IIIIII"))
+
+	# Payload module header struct:
+	#	Number of parameters (32 bit)
+	#	Reserved (32 bit)
+	plModStruct = struct.Struct(str(">II"))
+
+	# awlSources: List of AwlSource()s
+	# symTabSources: List of SymTabSource()s
+	# hwMods: List of tuples: (modName, parametersDict)
+	# libSelections: List of AwlLibEntrySelection()s
+	def __init__(self, awlSources, symTabSources, hwMods, libSelections):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_IDENTS)
+		self.awlSources = awlSources
+		self.symTabSources = symTabSources
+		self.hwMods = hwMods
+		self.libSelections = libSelections
+
+	def toBytes(self):
+		payload = [ self.plHdrStruct.pack(len(self.awlSources),
+						  len(self.symTabSources),
+						  len(self.hwMods),
+						  len(self.libSelections),
+						  0, 0), ]
+		for src in self.awlSources:
+			payload.append(self.packString(src.name))
+			payload.append(self.packString(src.filepath))
+			payload.append(self.packBytes(src.identHash))
+		for src in self.symTabSources:
+			payload.append(self.packString(src.name))
+			payload.append(self.packString(src.filepath))
+			payload.append(self.packBytes(src.identHash))
+		for modName, parametersDict in self.hwMods:
+			payload.append(self.plModStruct.pack(len(parametersDict), 0))
+			payload.append(self.packString(modName))
+			for pName, pVal in parametersDict.items():
+				payload.append(self.packString(pName))
+				payload.append(self.packString(pVal))
+		for libSel in self.libSelections:
+			payload.append(AwlSimMessage_LOAD_LIB.packLibSelection(libSel))
+		payload = b''.join(payload)
+		return AwlSimMessage.toBytes(self, len(payload)) + payload
+
+	@classmethod
+	def fromBytes(cls, payload):
+		try:
+			awlSources = []
+			symTabSources = []
+			hwMods = []
+			libSelections = []
+			offset = 0
+			nrAwl, nrSym, nrHw, nrLib, _a, _b  = cls.plHdrStruct.unpack_from(
+								payload, offset)
+			offset += cls.plHdrStruct.size
+			for i in range(nrAwl):
+				name, count = cls.unpackString(payload, offset)
+				offset += count
+				path, count = cls.unpackString(payload, offset)
+				offset += count
+				identHash, count = cls.unpackBytes(payload, offset)
+				offset += count
+				src = AwlSource(name, path, None)
+				src.identHash = identHash # Force hash
+				awlSources.append(src)
+			for i in range(nrSym):
+				name, count = cls.unpackString(payload, offset)
+				offset += count
+				path, count = cls.unpackString(payload, offset)
+				offset += count
+				identHash, count = cls.unpackBytes(payload, offset)
+				offset += count
+				src = SymTabSource(name, path, None)
+				src.identHash = identHash # Force hash
+				symTabSources.append(src)
+			for i in range(nrHw):
+				nrParam, _unused = cls.plModStruct.unpack_from(
+								payload, offset)
+				offset += cls.plModStruct.size
+				modName, count = cls.unpackString(payload, offset)
+				offset += count
+				params = {}
+				for i in range(nrParam):
+					pName, count = cls.unpackString(payload, offset)
+					offset += count
+					pVal, count = cls.unpackString(payload, offset)
+					offset += count
+					params[pName] = pVal
+				hwMods.append( (modName, params) )
+			for i in range(nrLib):
+				libSel, offset = AwlSimMessage_LOAD_LIB.unpackLibSelection(
+						payload, offset)
+				libSelections.append(libSel)
+		except (ValueError, struct.error, AwlSimError) as e:
+			raise TransferError("IDENTS: Invalid data format")
+		return cls(awlSources = awlSources,
+			   symTabSources = symTabSources,
+			   hwMods = hwMods,
+			   libSelections = libSelections)
+
+class AwlSimMessageTransceiver(object):
 	id2class = {
 		AwlSimMessage.MSG_ID_REPLY		: AwlSimMessage_REPLY,
 		AwlSimMessage.MSG_ID_EXCEPTION		: AwlSimMessage_EXCEPTION,
@@ -680,9 +866,11 @@ class AwlSimMessageTransceiver(object):
 		AwlSimMessage.MSG_ID_RESET		: AwlSimMessage_RESET,
 		AwlSimMessage.MSG_ID_SHUTDOWN		: AwlSimMessage_SHUTDOWN,
 		AwlSimMessage.MSG_ID_RUNSTATE		: AwlSimMessage_RUNSTATE,
+		AwlSimMessage.MSG_ID_GET_RUNSTATE	: AwlSimMessage_GET_RUNSTATE,
 		AwlSimMessage.MSG_ID_LOAD_SYMTAB	: AwlSimMessage_LOAD_SYMTAB,
 		AwlSimMessage.MSG_ID_LOAD_CODE		: AwlSimMessage_LOAD_CODE,
 		AwlSimMessage.MSG_ID_LOAD_HW		: AwlSimMessage_LOAD_HW,
+		AwlSimMessage.MSG_ID_LOAD_LIB		: AwlSimMessage_LOAD_LIB,
 		AwlSimMessage.MSG_ID_SET_OPT		: AwlSimMessage_SET_OPT,
 		AwlSimMessage.MSG_ID_CPUDUMP		: AwlSimMessage_CPUDUMP,
 		AwlSimMessage.MSG_ID_MAINTREQ		: AwlSimMessage_MAINTREQ,
@@ -692,7 +880,8 @@ class AwlSimMessageTransceiver(object):
 		AwlSimMessage.MSG_ID_MEMORY		: AwlSimMessage_MEMORY,
 		AwlSimMessage.MSG_ID_INSNSTATE		: AwlSimMessage_INSNSTATE,
 		AwlSimMessage.MSG_ID_INSNSTATE_CONFIG	: AwlSimMessage_INSNSTATE_CONFIG,
-		AwlSimMessage.MSG_ID_LOAD_LIB		: AwlSimMessage_LOAD_LIB,
+		AwlSimMessage.MSG_ID_GET_IDENTS		: AwlSimMessage_GET_IDENTS,
+		AwlSimMessage.MSG_ID_IDENTS		: AwlSimMessage_IDENTS,
 	}
 
 	def __init__(self, sock, peerInfoString):
@@ -719,19 +908,13 @@ class AwlSimMessageTransceiver(object):
 				self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 			self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)
 			self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192)
-		except socket.error as e:
+		except SocketErrors as e:
 			raise AwlSimError("Failed to initialize socket: %s" % str(e))
 
 	def shutdown(self):
 		if self.sock:
-			try:
-				self.sock.shutdown(socket.SHUT_RDWR)
-			except socket.error as e:
-				pass
-			try:
-				self.sock.close()
-			except socket.error as e:
-				pass
+			CALL_NOEX(self.sock.shutdown, socket.SHUT_RDWR)
+			CALL_NOEX(self.sock.close)
 			self.sock = None
 
 	def send(self, msg, timeout=None):
@@ -746,12 +929,10 @@ class AwlSimMessageTransceiver(object):
 		while offset < len(data):
 			try:
 				offset += self.sock.send(data[offset : ])
-			except (socket.error, BlockingIOError) as e:
-				if e.errno != errno.EAGAIN and\
-				   e.errno != errno.EWOULDBLOCK and\
-				   not isinstance(e, BlockingIOError) and\
-				   not isinstance(e, socket.timeout):
-					raise TransferError(str(e))
+			except SocketErrors as e:
+				transferError = TransferError(None, parentException = e)
+				if transferError.reason != TransferError.REASON_BLOCKING:
+					raise transferError
 
 	def receive(self, timeout=0.0):
 		if timeout != self.__timeout:
@@ -762,15 +943,15 @@ class AwlSimMessageTransceiver(object):
 		if len(self.buf) < hdrLen:
 			try:
 				data = self.sock.recv(hdrLen - len(self.buf))
-			except (socket.error, BlockingIOError) as e:
-				if e.errno == errno.EAGAIN or\
-				   e.errno == errno.EWOULDBLOCK or\
-				   isinstance(e, BlockingIOError):
+			except SocketErrors as e:
+				transferError = TransferError(None, parentException = e)
+				if transferError.reason == TransferError.REASON_BLOCKING:
 					return None
-				raise
+				raise transferError
 			if not data:
 				# The remote end closed the connection
-				raise self.RemoteEndDied()
+				raise TransferError(None,
+					reason = TransferError.REASON_REMOTEDIED)
 			self.buf += data
 			if len(self.buf) < hdrLen:
 				return None
@@ -790,7 +971,8 @@ class AwlSimMessageTransceiver(object):
 			data = self.sock.recv(hdrLen + self.payloadLen - len(self.buf))
 			if not data:
 				# The remote end closed the connection
-				raise self.RemoteEndDied()
+				raise TransferError(None,
+					reason = TransferError.REASON_REMOTEDIED)
 			self.buf += data
 			if len(self.buf) < hdrLen + self.payloadLen:
 				return None
