@@ -51,9 +51,10 @@ class AwlSimServer(object):
 	ENV_MAGIC	= "AWLSIM_CORESERVER_MAGIC"
 
 	EnumGen.start
-	STATE_INIT	= EnumGen.item
-	STATE_RUN	= EnumGen.item
-	STATE_EXIT	= EnumGen.item
+	STATE_INIT		= EnumGen.item
+	STATE_RUN		= EnumGen.item
+	STATE_MAINTENANCE	= EnumGen.item
+	STATE_EXIT		= EnumGen.item
 	EnumGen.end
 
 	# Command mask bits
@@ -67,6 +68,9 @@ class AwlSimServer(object):
 			self.socket = sock
 			self.transceiver = AwlSimMessageTransceiver(sock, peerInfoString)
 
+			# Broken-flag. Set, if connection breaks.
+			self.broken = False
+
 			# CPU-dump
 			self.dumpInterval = 0
 			self.nextDump = 0
@@ -109,17 +113,11 @@ class AwlSimServer(object):
 				return False
 			sock = socket.socket(family, socktype)
 			sock.bind(sockaddr)
-		except socket.error as e:
+		except SocketErrors as e:
 			result = False
 		if sock:
-			try:
-				sock.shutdown(socket.SHUT_RDWR)
-			except socket.error as e:
-				pass
-			try:
-				sock.close()
-			except socket.error as e:
-				pass
+			CALL_NOEX(sock.shutdown, socket.SHUT_RDWR)
+			CALL_NOEX(sock.close)
 		return result
 
 	@classmethod
@@ -137,7 +135,7 @@ class AwlSimServer(object):
 		env[AwlSimServer.ENV_MAGIC]		= AwlSimServer.ENV_MAGIC
 		env["AWLSIM_CORESERVER_HOST"]		= str(listenHost)
 		env["AWLSIM_CORESERVER_PORT"]		= str(int(listenPort))
-		env["AWLSIM_CORESERVER_LOGLEVEL"]	= str(Logging.getLoglevel())
+		env["AWLSIM_CORESERVER_LOGLEVEL"]	= str(Logging.loglevel)
 		env["AWLSIM_CORESERVER_CMDMSK"]		= str(int(commandMask))
 
 		if forkServerProcess:
@@ -188,12 +186,27 @@ class AwlSimServer(object):
 	def __init__(self):
 		self.state = -1
 		self.__setRunState(self.STATE_INIT)
+
+		self.__nextStats = 0
 		self.commandMask = 0
-		self.sim = None
 		self.socket = None
 		self.unixSockPath = None
 		self.clients = []
 
+		self.sim = None
+		self.__resetSources()
+
+	def __resetSources(self):
+		# List of loaded AwlSource()s
+		self.loadedAwlSources = []
+		# List of loaded SymTabSource()s
+		self.loadedSymTabSources = []
+		# List of tuples of loaded hardware modules:
+		#   (hwModName, parameterDict)
+		self.loadedHwModules = []
+		# List of loaded AwlLibEntrySelection()s
+		self.loadedLibSelections = []
+
 	def runFromEnvironment(self, env=None):
 		"""Run the server.
 		Configuration is passed via environment variables in 'env'.
@@ -237,7 +250,9 @@ class AwlSimServer(object):
 		# Make a shortcut variable for RUN
 		self.__running = bool(runstate == self.STATE_RUN)
 
-		self.__insnSerial = 0
+		if runstate == self.STATE_RUN or\
+		   runstate == self.STATE_INIT:
+			self.__insnSerial = 0
 
 	def __rebuildSelectReadList(self):
 		rlist = [ self.socket ]
@@ -248,10 +263,16 @@ class AwlSimServer(object):
 		now = self.sim.cpu.now
 		if any(c.dumpInterval and now >= c.nextDump for c in self.clients):
 			msg = AwlSimMessage_CPUDUMP(str(self.sim.cpu))
+			broken = False
 			for client in self.clients:
 				if client.dumpInterval and now >= client.nextDump:
 					client.nextDump = now + client.dumpInterval / 1000.0
-					client.transceiver.send(msg)
+					try:
+						client.transceiver.send(msg)
+					except TransferError as e:
+						client.broken = broken = True
+			if broken:
+				self.__removeBrokenClients()
 
 	def __cpuPostInsnCallback(self, callStackElement, userData):
 		try:
@@ -260,6 +281,7 @@ class AwlSimServer(object):
 			return
 		cpu, sourceId, lineNr, msg =\
 			self.sim.cpu, insn.getSourceId(), insn.getLineNr(), None
+		broken = False
 		for client in self.clients:
 			try:
 				if lineNr not in client.insnStateDump_enabledLines[sourceId]:
@@ -281,12 +303,37 @@ class AwlSimServer(object):
 					cpu.ar2.get(),
 					cpu.dbRegister.index & 0xFFFF,
 					cpu.diRegister.index & 0xFFFF)
-			client.transceiver.send(msg)
+			try:
+				client.transceiver.send(msg)
+			except TransferError as e:
+				client.broken = broken = True
+		if broken:
+			self.__removeBrokenClients()
 		self.__insnSerial += 1
 
+	def __printCpuStats(self):
+		cpu = self.sim.cpu
+		if cpu.insnPerSecond:
+			usPerInsn = "%.03f" % ((1.0 / cpu.insnPerSecond) * 1000000)
+		else:
+			usPerInsn = "-/-"
+		printVerbose("[CPU] "
+			"%d stmt/s (= %s us/stmt); %.01f stmt/cycle" %\
+			(int(round(cpu.insnPerSecond)),
+			 usPerInsn,
+			 cpu.avgInsnPerCycle))
+
 	def __cpuCycleExitCallback(self, userData):
+		# Reset instruction dump serial number
 		self.__insnSerial = 0
 
+		# Print CPU stats, if requested.
+		if Logging.loglevel >= Logging.LOG_VERBOSE:
+			now = self.sim.cpu.now
+			if now >= self.__nextStats:
+				self.__nextStats = now + 1.0
+				self.__printCpuStats()
+
 	def __updateCpuBlockExitCallback(self):
 		if any(c.dumpInterval for c in self.clients):
 			self.sim.cpu.setBlockExitCallback(self.__cpuBlockExitCallback, None)
@@ -300,74 +347,114 @@ class AwlSimServer(object):
 			self.sim.cpu.setPostInsnCallback(None)
 
 	def __updateCpuCycleExitCallback(self):
-		if any(c.insnStateDump_enabledLines for c in self.clients):
+		if any(c.insnStateDump_enabledLines for c in self.clients) or\
+		   Logging.loglevel >= Logging.LOG_VERBOSE:
 			self.sim.cpu.setCycleExitCallback(self.__cpuCycleExitCallback, None)
 		else:
 			self.sim.cpu.setCycleExitCallback(None)
 
+	def __updateCpuCallbacks(self):
+		self.__updateCpuBlockExitCallback()
+		self.__updateCpuPostInsnCallback()
+		self.__updateCpuCycleExitCallback()
+
 	def __rx_PING(self, client, msg):
+		printDebug("Received message: PING")
 		client.transceiver.send(AwlSimMessage_PONG())
 
 	def __rx_PONG(self, client, msg):
-		printInfo("AwlSimServer: Received PONG")
+		printInfo("Received message: PONG")
 
 	def __rx_RESET(self, client, msg):
+		printVerbose("Resetting CPU.")
 		status = AwlSimMessage_REPLY.STAT_OK
 		self.__setRunState(self.STATE_INIT)
 		self.sim.reset()
+		self.__resetSources()
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_SHUTDOWN(self, client, msg):
+		printDebug("Received message: SHUTDOWN")
 		status = AwlSimMessage_REPLY.STAT_FAIL
 		if self.commandMask & AwlSimServer.CMDMSK_SHUTDOWN:
-			printInfo("AwlSimServer: Exiting due to shutdown command")
+			printInfo("Exiting due to shutdown command")
 			self.__setRunState(self.STATE_EXIT)
 			status = AwlSimMessage_REPLY.STAT_OK
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_RUNSTATE(self, client, msg):
+		printDebug("Received message: RUNSTATE %d" % msg.runState)
 		status = AwlSimMessage_REPLY.STAT_OK
 		if msg.runState == msg.STATE_STOP:
+			printVerbose("Putting CPU into STOP state.")
 			self.__setRunState(self.STATE_INIT)
 		elif msg.runState == msg.STATE_RUN:
-			self.sim.startup()
-			self.__setRunState(self.STATE_RUN)
+			if self.state == self.STATE_RUN:
+				pass
+			elif self.state == self.STATE_INIT:
+				printVerbose("CPU startup (OB 10x).")
+				self.sim.startup()
+				printVerbose("Putting CPU into RUN state.")
+				self.__setRunState(self.STATE_RUN)
+			elif self.state == self.STATE_MAINTENANCE:
+				printVerbose("Putting CPU into RUN state.")
+				self.__setRunState(self.STATE_RUN)
+			else:
+				status = AwlSimMessage_REPLY.STAT_FAIL
 		else:
 			status = AwlSimMessage_REPLY.STAT_FAIL
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
+	def __rx_GET_RUNSTATE(self, client, msg):
+		printDebug("Received message: GET_RUNSTATE")
+		reply = AwlSimMessage_RUNSTATE(
+			AwlSimMessage_RUNSTATE.STATE_RUN\
+			if self.state == self.STATE_RUN else\
+			AwlSimMessage_RUNSTATE.STATE_STOP
+		)
+		client.transceiver.send(reply)
+
 	def __rx_LOAD_CODE(self, client, msg):
+		printDebug("Received message: LOAD_CODE")
 		status = AwlSimMessage_REPLY.STAT_OK
 		parser = AwlParser()
 		parser.parseSource(msg.source)
 		self.__setRunState(self.STATE_INIT)
 		self.sim.load(parser.getParseTree())
+		self.loadedAwlSources.append(msg.source)
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_LOAD_SYMTAB(self, client, msg):
+		printDebug("Received message: LOAD_SYMTAB")
 		status = AwlSimMessage_REPLY.STAT_OK
 		symbolTable = SymTabParser.parseSource(msg.source,
 					autodetectFormat = True,
 					mnemonics = self.sim.cpu.getSpecs().getMnemonics())
 		self.__setRunState(self.STATE_INIT)
 		self.sim.loadSymbolTable(symbolTable)
+		self.loadedSymTabSources.append(msg.source)
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_LOAD_HW(self, client, msg):
+		printDebug("Received message: LOAD_HW")
 		status = AwlSimMessage_REPLY.STAT_OK
 		printInfo("Loading hardware module '%s'..." % msg.name)
 		hwClass = self.sim.loadHardwareModule(msg.name)
 		self.sim.registerHardwareClass(hwClass = hwClass,
 					       parameters = msg.paramDict)
+		self.loadedHwModules.append( (msg.name, msg.paramDict) )
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_LOAD_LIB(self, client, msg):
+		printDebug("Received message: LOAD_LIB")
 		status = AwlSimMessage_REPLY.STAT_OK
 		self.__setRunState(self.STATE_INIT)
 		self.sim.loadLibraryBlock(msg.libSelection)
+		self.loadedLibSelections.append(msg.libSelection)
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_SET_OPT(self, client, msg):
+		printDebug("Received message: SET_OPT %s" % msg.name)
 		status = AwlSimMessage_REPLY.STAT_OK
 
 		if msg.name == "loglevel":
@@ -382,7 +469,7 @@ class AwlSimServer(object):
 				client.nextDump = self.sim.cpu.now
 			else:
 				client.nextDump = None
-			self.__updateCpuBlockExitCallback()
+			self.__updateCpuCallbacks()
 		elif msg.name == "cycle_time_limit":
 			self.sim.cpu.setCycleTimeLimit(msg.getFloatValue())
 		elif msg.name == "runtime_limit":
@@ -393,15 +480,18 @@ class AwlSimServer(object):
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_GET_CPUSPECS(self, client, msg):
+		printDebug("Received message: GET_CPUSPECS")
 		reply = AwlSimMessage_CPUSPECS(self.sim.cpu.getSpecs())
 		client.transceiver.send(reply)
 
 	def __rx_CPUSPECS(self, client, msg):
+		printDebug("Received message: CPUSPECS")
 		status = AwlSimMessage_REPLY.STAT_OK
 		self.sim.cpu.getSpecs().assignFrom(msg.cpuspecs)
 		client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_REQ_MEMORY(self, client, msg):
+		printDebug("Received message: REQ_MEMORY")
 		client.memReadRequestMsg = AwlSimMessage_MEMORY(0, msg.memAreas)
 		client.repetitionFactor = msg.repetitionFactor
 		client.repetitionCount = client.repetitionFactor
@@ -411,6 +501,7 @@ class AwlSimServer(object):
 			)
 
 	def __rx_MEMORY(self, client, msg):
+		printDebug("Received message: MEMORY")
 		cpu = self.sim.cpu
 		status = AwlSimMessage_REPLY.STAT_OK
 		for memArea in msg.memAreas:
@@ -429,56 +520,82 @@ class AwlSimServer(object):
 			client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	def __rx_INSNSTATE_CONFIG(self, client, msg):
+		printDebug("Received message: INSNSTATE_CONFIG")
 		status = AwlSimMessage_REPLY.STAT_OK
 		if msg.flags & (msg.FLG_CLEAR | msg.FLG_CLEAR_ONLY):
 			client.insnStateDump_enabledLines = {}
 		if not (msg.flags & msg.FLG_CLEAR_ONLY):
 			rnge = range(msg.fromLine, msg.toLine + 1)
 			client.insnStateDump_enabledLines[msg.sourceId] = rnge
-		self.__updateCpuPostInsnCallback()
-		self.__updateCpuCycleExitCallback()
+		self.__updateCpuCallbacks()
 		if msg.flags & msg.FLG_SYNC:
 			client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
+	def __rx_GET_IDENTS(self, client, msg):
+		printDebug("Received message: GET_IDENTS")
+		awlSrcs = symSrcs = hwMods = libSels = ()
+		if msg.getFlags & msg.GET_AWLSRCS:
+			awlSrcs = self.loadedAwlSources
+		if msg.getFlags & msg.GET_SYMTABSRCS:
+			symSrcs = self.loadedSymTabSources
+		if msg.getFlags & msg.GET_HWMODS:
+			hwMods = self.loadedHwModules
+		if msg.getFlags & msg.GET_LIBSELS:
+			libSels = self.loadedLibSelections
+		reply = AwlSimMessage_IDENTS(awlSrcs, symSrcs,
+					     hwMods, libSels)
+		client.transceiver.send(reply)
+
 	__msgRxHandlers = {
 		AwlSimMessage.MSG_ID_PING		: __rx_PING,
 		AwlSimMessage.MSG_ID_PONG		: __rx_PONG,
 		AwlSimMessage.MSG_ID_RESET		: __rx_RESET,
 		AwlSimMessage.MSG_ID_SHUTDOWN		: __rx_SHUTDOWN,
 		AwlSimMessage.MSG_ID_RUNSTATE		: __rx_RUNSTATE,
+		AwlSimMessage.MSG_ID_GET_RUNSTATE	: __rx_GET_RUNSTATE,
 		AwlSimMessage.MSG_ID_LOAD_CODE		: __rx_LOAD_CODE,
 		AwlSimMessage.MSG_ID_LOAD_SYMTAB	: __rx_LOAD_SYMTAB,
 		AwlSimMessage.MSG_ID_LOAD_HW		: __rx_LOAD_HW,
+		AwlSimMessage.MSG_ID_LOAD_LIB		: __rx_LOAD_LIB,
 		AwlSimMessage.MSG_ID_SET_OPT		: __rx_SET_OPT,
 		AwlSimMessage.MSG_ID_GET_CPUSPECS	: __rx_GET_CPUSPECS,
 		AwlSimMessage.MSG_ID_CPUSPECS		: __rx_CPUSPECS,
 		AwlSimMessage.MSG_ID_REQ_MEMORY		: __rx_REQ_MEMORY,
 		AwlSimMessage.MSG_ID_MEMORY		: __rx_MEMORY,
 		AwlSimMessage.MSG_ID_INSNSTATE_CONFIG	: __rx_INSNSTATE_CONFIG,
-		AwlSimMessage.MSG_ID_LOAD_LIB		: __rx_LOAD_LIB,
+		AwlSimMessage.MSG_ID_GET_IDENTS		: __rx_GET_IDENTS,
 	}
 
+	def __clientCommTransferError(self, exception, client):
+		if exception.reason == exception.REASON_REMOTEDIED:
+			printInfo("Client '%s' died" %\
+				  client.transceiver.peerInfoString)
+		else:
+			printInfo("Client '%s' data "
+				"transfer error:\n%s" %\
+				(client.transceiver.peerInfoString,
+				 str(exception)))
+		self.__clientRemove(client)
+
 	def __handleClientComm(self, client):
 		try:
 			msg = client.transceiver.receive(0.0)
-		except AwlSimMessageTransceiver.RemoteEndDied as e:
-			printInfo("AwlSimServer: Client '%s' died" % client.transceiver.peerInfoString)
-			self.__clientRemove(client)
-			return
-		except (TransferError, socket.error) as e:
-			printInfo("AwlSimServer: Client '%s' data "
-				"transfer error:\n%s" %\
-				(client.transceiver.peerInfoString, str(e)))
+		except TransferError as e:
+			self.__clientCommTransferError(e, client)
 			return
 		if not msg:
 			return
 		try:
 			handler = self.__msgRxHandlers[msg.msgId]
 		except KeyError:
-			printInfo("AwlSimServer: Received unsupported "
+			printInfo("Received unsupported "
 				"message 0x%02X" % msg.msgId)
 			return
-		handler(self, client, msg)
+		try:
+			handler(self, client, msg)
+		except TransferError as e:
+			self.__clientCommTransferError(e, client)
+			return
 
 	def __handleCommunication(self):
 		while 1:
@@ -497,6 +614,7 @@ class AwlSimServer(object):
 				self.__handleClientComm(client)
 
 	def __handleMemReadReqs(self):
+		broken = False
 		for client in self.clients:
 			if not client.memReadRequestMsg:
 				continue
@@ -518,14 +636,20 @@ class AwlSimServer(object):
 							# This is a serious fault.
 							# Re-raise the exception.
 							raise
-				client.transceiver.send(client.memReadRequestMsg)
+				try:
+					client.transceiver.send(client.memReadRequestMsg)
+				except TransferError as e:
+					client.broken = broken = True
 				client.repetitionCount = client.repetitionFactor
 				if not client.repetitionFactor:
 					self.memReadRequestMsg = None
+		if broken:
+			self.__removeBrokenClients()
 
 	def run(self, host, port, commandMask):
 		"""Run the server on 'host':'port'."""
 
+		Logging.setPrefix("AwlSimServer: ")
 		self.commandMask = commandMask
 
 		self.__listen(host, port)
@@ -533,13 +657,17 @@ class AwlSimServer(object):
 
 		self.sim = AwlSim()
 		nextComm = 0.0
+		self.__nextStats = self.sim.cpu.now
+		self.__updateCpuCallbacks()
 
 		while self.state != self.STATE_EXIT:
 			try:
 				sim = self.sim
 
-				if self.state == self.STATE_INIT:
-					while self.state == self.STATE_INIT:
+				if self.state in (self.STATE_INIT,
+						  self.STATE_MAINTENANCE):
+					while self.state in (self.STATE_INIT,
+							     self.STATE_MAINTENANCE):
 						self.__handleCommunication()
 						time.sleep(0.01)
 					continue
@@ -557,10 +685,17 @@ class AwlSimServer(object):
 					try:
 						client.transceiver.send(msg)
 					except TransferError as e:
-						printError("AwlSimServer: Failed to forward "
+						printError("Failed to forward "
 							   "exception to client.")
+						client.broken = True
+				self.__removeBrokenClients()
 				self.__setRunState(self.STATE_INIT)
 			except MaintenanceRequest as e:
+				# Put the CPU into maintenance mode.
+				# This will halt the CPU until a client
+				# sets it into RUN or STOP again.
+				self.__setRunState(self.STATE_MAINTENANCE)
+				# Send the maintenance message.
 				try:
 					if self.clients:
 						# Forward it to the first client
@@ -569,8 +704,8 @@ class AwlSimServer(object):
 				except TransferError as e:
 					pass
 			except TransferError as e:
-				printError("AwlSimServer: Transfer error: " + str(e))
-				self.__setRunState(self.STATE_INIT)
+				# This should be caught earlier.
+				printError("Uncaught transfer error: " + str(e))
 
 	def __listen(self, host, port):
 		"""Listen on 'host':'port'."""
@@ -583,13 +718,13 @@ class AwlSimServer(object):
 				readableSockaddr = sockaddr
 			else:
 				readableSockaddr = "%s:%d" % (sockaddr[0], sockaddr[1])
-			printInfo("AwlSimServer: Listening on %s..." % readableSockaddr)
+			printInfo("Listening on %s..." % readableSockaddr)
 			sock = socket.socket(family, socktype)
 			sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 			sock.setblocking(False)
 			sock.bind(sockaddr)
 			sock.listen(5)
-		except socket.error as e:
+		except SocketErrors as e:
 			raise AwlSimError("AwlSimServer: Failed to create server "
 				"socket: " + str(e))
 		self.socket = sock
@@ -607,13 +742,12 @@ class AwlSimServer(object):
 				peerInfoString = self.unixSockPath
 			else:
 				peerInfoString = "%s:%d" % addrInfo[:2]
-		except (socket.error, BlockingIOError) as e:
-			if isinstance(e, BlockingIOError) or\
-			   e.errno == errno.EWOULDBLOCK or\
-			   e.errno == errno.EAGAIN:
+		except SocketErrors as e:
+			transferError = TransferError(None, parentException = e)
+			if transferError.reason == transferError.REASON_BLOCKING:
 				return None
 			raise AwlSimError("AwlSimServer: accept() failed: %s" % str(e))
-		printInfo("AwlSimServer: Client '%s' connected" % peerInfoString)
+		printInfo("Client '%s' connected" % peerInfoString)
 
 		client = self.Client(clientSock, peerInfoString)
 		self.__clientAdd(client)
@@ -627,12 +761,17 @@ class AwlSimServer(object):
 	def __clientRemove(self, client):
 		self.clients.remove(client)
 		self.__rebuildSelectReadList()
+		self.__updateCpuCallbacks()
+
+	def __removeBrokenClients(self):
+		for client in [ c for c in self.clients if c.broken ]:
+			self.__clientRemove(client)
 
 	def close(self):
 		"""Closes all client sockets and the main socket."""
 
 		if self.socket:
-			printInfo("AwlSimServer: Shutting down.")
+			printInfo("Shutting down.")
 
 		if self.sim:
 			self.sim.shutdown()
@@ -645,14 +784,8 @@ class AwlSimServer(object):
 		self.clients = []
 
 		if self.socket:
-			try:
-				self.socket.shutdown(socket.SHUT_RDWR)
-			except socket.error as e:
-				pass
-			try:
-				self.socket.close()
-			except socket.error as e:
-				pass
+			CALL_NOEX(self.socket.shutdown, socket.SHUT_RDWR)
+			CALL_NOEX(self.socket.close)
 			self.socket = None
 		if self.unixSockPath:
 			try:
@@ -662,7 +795,7 @@ class AwlSimServer(object):
 			self.unixSockPath = None
 
 	def signalHandler(self, sig, frame):
-		printInfo("AwlSimServer: Received signal %d" % sig)
+		printInfo("Received signal %d" % sig)
 		if sig in (signal.SIGTERM, signal.SIGINT):
 			self.__setRunState(self.STATE_EXIT)
 
