@@ -38,13 +38,18 @@ class StateWindow(QWidget):
 		pixmap = QPixmap(16, 16)
 		pixmap.fill(QColor(0, 0, 192))
 		self.setWindowIcon(QIcon(pixmap))
-		self.client = client
+		self.__client = client
 
 	def update(self):
-		size, hint = self.size(), self.minimumSizeHint()
+		parent = self.parent()
+		if isinstance(parent, StateMdiSubWindow):
+			widget = parent
+		else:
+			widget = self
+		size, hint = widget.size(), widget.minimumSizeHint()
 		if size.width() < hint.width() or\
 		   size.height() < hint.height():
-			self.resize(hint)
+			widget.resize(hint)
 
 	# Get a list of MemoryArea instances for the memory
 	# areas covered by this window.
@@ -66,6 +71,18 @@ class StateWindow(QWidget):
 		ev.accept()
 		self.deleteLater()
 
+	# Send a request to the CPU to write memory.
+	def _writeCpuMemory(self, memAreas):
+		try:
+			self.__client.writeMemory(memAreas, sync = False)
+		except AwlSimError as e:
+			MessageBox.handleAwlSimError(self,
+				"Failed to write to memory", e)
+		except MaintenanceRequest as e:
+			# writeMemory is asynchronous.
+			# So this should not happen.
+			assert(0)
+
 class State_CPU(StateWindow):
 	def __init__(self, client, parent=None):
 		StateWindow.__init__(self, client, parent)
@@ -109,11 +126,11 @@ class AbstractDisplayWidget(QWidget):
 
 	changed = Signal()
 
-	def __init__(self, client, addrSpace, addr, width, db, parent=None):
+	def __init__(self, stateWindow, addrSpace, addr, width, db, parent=None):
 		QWidget.__init__(self, parent)
 		self.setLayout(QGridLayout(self))
 
-		self.client = client
+		self.stateWindow = stateWindow
 		self.addrSpace = addrSpace
 		self.addr = addr
 		self.width = width
@@ -139,10 +156,10 @@ class AbstractDisplayWidget(QWidget):
 			self.setPalette(pal)
 
 class BitDisplayWidget(AbstractDisplayWidget):
-	def __init__(self, client, addrSpace, addr, width, db,
+	def __init__(self, stateWindow, addrSpace, addr, width, db,
 		     parent=None,
 		     displayPushButtons=True):
-		AbstractDisplayWidget.__init__(self, client, addrSpace,
+		AbstractDisplayWidget.__init__(self, stateWindow, addrSpace,
 					       addr, width, db, parent)
 
 		self.cbs = {}
@@ -195,8 +212,8 @@ class BitDisplayWidget(AbstractDisplayWidget):
 			value >>= 1
 
 class NumberDisplayWidget(AbstractDisplayWidget):
-	def __init__(self, client, base, addrSpace, addr, width, db, parent=None):
-		AbstractDisplayWidget.__init__(self, client, addrSpace,
+	def __init__(self, stateWindow, base, addrSpace, addr, width, db, parent=None):
+		AbstractDisplayWidget.__init__(self, stateWindow, addrSpace,
 					       addr, width, db, parent)
 
 		self.base = base
@@ -271,23 +288,23 @@ class NumberDisplayWidget(AbstractDisplayWidget):
 		self.line.setText(string)
 
 class HexDisplayWidget(NumberDisplayWidget):
-	def __init__(self, client, addrSpace, addr, width, db, parent=None):
-		NumberDisplayWidget.__init__(self, client, 16, addrSpace,
+	def __init__(self, stateWindow, addrSpace, addr, width, db, parent=None):
+		NumberDisplayWidget.__init__(self, stateWindow, 16, addrSpace,
 					     addr, width, db, parent)
 
 class DecDisplayWidget(NumberDisplayWidget):
-	def __init__(self, client, addrSpace, addr, width, db, parent=None):
-		NumberDisplayWidget.__init__(self, client, 10, addrSpace,
+	def __init__(self, stateWindow, addrSpace, addr, width, db, parent=None):
+		NumberDisplayWidget.__init__(self, stateWindow, 10, addrSpace,
 					     addr, width, db, parent)
 
 class BinDisplayWidget(NumberDisplayWidget):
-	def __init__(self, client, addrSpace, addr, width, db, parent=None):
-		NumberDisplayWidget.__init__(self, client, 2, addrSpace,
+	def __init__(self, stateWindow, addrSpace, addr, width, db, parent=None):
+		NumberDisplayWidget.__init__(self, stateWindow, 2, addrSpace,
 					     addr, width, db, parent)
 
 class RealDisplayWidget(AbstractDisplayWidget):
-	def __init__(self, client, addrSpace, addr, width, db, parent=None):
-		AbstractDisplayWidget.__init__(self, client, addrSpace,
+	def __init__(self, stateWindow, addrSpace, addr, width, db, parent=None):
+		AbstractDisplayWidget.__init__(self, stateWindow, addrSpace,
 					       addr, width, db, parent)
 
 		self.__currentValue = -1
@@ -383,7 +400,7 @@ class State_Mem(StateWindow):
 		self.widthCombo.currentIndexChanged.connect(self.rebuild)
 		self.fmtCombo.currentIndexChanged.connect(self.rebuild)
 
-		self.__changeBlocked = 0
+		self.__changeBlocked = Blocker()
 		self.rebuild()
 
 	def rebuild(self):
@@ -422,28 +439,28 @@ class State_Mem(StateWindow):
 		self.setWindowTitle(longName)
 
 		if fmt == "cb":
-			self.contentWidget = BitDisplayWidget(self.client,
+			self.contentWidget = BitDisplayWidget(self,
 							      self.addrSpace,
 							      addr, width, db, self,
 							      displayPushButtons=True)
 			self.contentLayout.addWidget(self.contentWidget)
 		elif fmt == "hex":
-			self.contentWidget = HexDisplayWidget(self.client,
+			self.contentWidget = HexDisplayWidget(self,
 							      self.addrSpace,
 							      addr, width, db, self)
 			self.contentLayout.addWidget(self.contentWidget)
 		elif fmt == "dec":
-			self.contentWidget = DecDisplayWidget(self.client,
+			self.contentWidget = DecDisplayWidget(self,
 							      self.addrSpace,
 							      addr, width, db, self)
 			self.contentLayout.addWidget(self.contentWidget)
 		elif fmt == "bin":
-			self.contentWidget = BinDisplayWidget(self.client,
+			self.contentWidget = BinDisplayWidget(self,
 							      self.addrSpace,
 							      addr, width, db, self)
 			self.contentLayout.addWidget(self.contentWidget)
 		elif fmt == "real":
-			self.contentWidget = RealDisplayWidget(self.client,
+			self.contentWidget = RealDisplayWidget(self,
 							       self.addrSpace,
 							       addr, width, db, self)
 			self.contentLayout.addWidget(self.contentWidget)
@@ -469,7 +486,7 @@ class State_Mem(StateWindow):
 		memArea.data = bytearray(width // 8)
 		WordPacker.toBytes(memArea.data, width, 0, value)
 
-		self.client.writeMemory((memArea,))
+		self._writeCpuMemory((memArea,))
 
 	def __makeMemoryArea(self):
 		dbIndex = 0
@@ -499,16 +516,15 @@ class State_Mem(StateWindow):
 			self.contentWidget.setEnabled(False)
 			return False
 
-		self.__changeBlocked += 1
-		addr = memArea.start
-		for value in memArea.data:
-			if addr > thisEnd:
-				break
-			if addr >= thisStart:
-				self.contentWidget.setByte(addr - thisStart,
-							   value)
-			addr += 1
-		self.__changeBlocked -= 1
+		with self.__changeBlocked:
+			addr = memArea.start
+			for value in memArea.data:
+				if addr > thisEnd:
+					break
+				if addr >= thisStart:
+					self.contentWidget.setByte(addr - thisStart,
+								   value)
+				addr += 1
 		self.update()
 		return True
 
@@ -552,7 +568,7 @@ class State_LCD(StateWindow):
 		self.endianCombo.currentIndexChanged.connect(self.rebuild)
 		self.fmtCombo.currentIndexChanged.connect(self.rebuild)
 
-		self.__changeBlocked = 0
+		self.__changeBlocked = Blocker()
 		self.rebuild()
 
 	def getDataWidth(self):
@@ -594,16 +610,15 @@ class State_LCD(StateWindow):
 			#TODO
 			return False
 
-		self.__changeBlocked += 1
-		addr = memArea.start
-		for value in memArea.data:
-			if addr > thisEnd:
-				break
-			if addr >= thisStart:
-				self.__setByte(addr - thisStart,
-					       value)
-			addr += 1
-		self.__changeBlocked -= 1
+		with self.__changeBlocked:
+			addr = memArea.start
+			for value in memArea.data:
+				if addr > thisEnd:
+					break
+				if addr >= thisStart:
+					self.__setByte(addr - thisStart,
+						       value)
+				addr += 1
 		self.update()
 		return True
 
@@ -670,10 +685,9 @@ class State_LCD(StateWindow):
 				assert(0)
 		else:
 			assert(0)
-		self.__changeBlocked += 1
-		self.lcd.setDigitCount(len(value))
-		self.lcd.display(value)
-		self.__changeBlocked -= 1
+		with self.__changeBlocked:
+			self.lcd.setDigitCount(len(value))
+			self.lcd.display(value)
 
 class _State_TimerCounter(StateWindow):
 	def __init__(self, client, memAreaType, parent=None):
@@ -704,7 +718,7 @@ class _State_TimerCounter(StateWindow):
 		self.valueEdit.valueChanged.connect(self.__newValueEntered)
 		self.resetButton.released.connect(self.reset)
 
-		self.__changeBlocked = 0
+		self.__changeBlocked = Blocker()
 		self.setMinimumWidth(310)
 		self.rebuild()
 
@@ -756,9 +770,8 @@ class _State_TimerCounter(StateWindow):
 		self.displayedStatus = status
 		self.__updateStatus()
 
-		self.__changeBlocked += 1
-		self.valueEdit.setText(text)
-		self.__changeBlocked -= 1
+		with self.__changeBlocked:
+			self.valueEdit.setText(text)
 
 		self.update()
 		return True
@@ -786,7 +799,7 @@ class _State_TimerCounter(StateWindow):
 		memArea = self.__makeMemoryArea()
 		memArea.data = bytearray(4)
 		WordPacker.toBytes(memArea.data, 32, 0, value)
-		self.client.writeMemory((memArea,))
+		self._writeCpuMemory((memArea,))
 
 class State_Timer(_State_TimerCounter):
 	def __init__(self, client, parent=None):
@@ -906,6 +919,18 @@ class State_Counter(_State_TimerCounter):
 			assert(0)
 		return text
 
-class StateWorkspace(QWorkspace):
-	def __init__(self, parent=None):
-		QWorkspace.__init__(self, parent)
+class StateMdiArea(QMdiArea):
+	pass
+
+class StateMdiSubWindow(QMdiSubWindow):
+	closed = Signal(QMdiSubWindow)
+
+	def __init__(self, childWidget):
+		QMdiSubWindow.__init__(self)
+		self.setWidget(childWidget)
+		childWidget.setParent(self)
+		self.setAttribute(Qt.WA_DeleteOnClose)
+
+	def closeEvent(self, ev):
+		self.closed.emit(self)
+		QMdiSubWindow.closeEvent(self, ev)
