@@ -26,11 +26,47 @@ from awlsim.gui.util import *
 from awlsim.gui.cpustate import *
 from awlsim.gui.awlsimclient import *
 from awlsim.gui.icons import *
+from awlsim.gui.linkconfig import *
 
 
+class RunState(QObject):
+	# Signal: Emitted, if the state changed.
+	# The parameter is 'self'.
+	stateChanged = Signal(QObject)
+
+	EnumGen.start
+	STATE_OFFLINE	= EnumGen.item
+	STATE_ONLINE	= EnumGen.item
+	STATE_LOAD	= EnumGen.item
+	STATE_RUN	= EnumGen.item
+	STATE_EXCEPTION	= EnumGen.item
+	EnumGen.end
+
+	def __init__(self):
+		QObject.__init__(self)
+		self.state = self.STATE_OFFLINE
+		self.setCoreDetails()
+
+	def __emitStateChanged(self):
+		self.stateChanged.emit(self)
+		QApplication.processEvents(QEventLoop.ExcludeUserInputEvents,
+					   1000)
+
+	def setState(self, newState):
+		if self.state != newState:
+			self.state = newState
+			self.__emitStateChanged()
+
+	def setCoreDetails(self, spawned=True,
+			   host=None, port=None):
+		self.spawned = spawned
+		self.host = host
+		self.port = port
+		self.__emitStateChanged()
+
 class ToolButton(QPushButton):
 	ICONSIZE	= (32, 32)
-	BTNFACT		= 1.3
+	BTNFACT		= 1.2
 
 	def __init__(self, iconName, description, parent=None):
 		QPushButton.__init__(self, parent)
@@ -46,9 +82,21 @@ class ToolButton(QPushButton):
 
 		self.setToolTip(description)
 
-class RunButton(ToolButton):
-	def __init__(self, parent=None):
-		ToolButton.__init__(self, "run", "", parent)
+class CheckableToolButton(ToolButton):
+	def __init__(self,
+		     checkedIconName, uncheckedIconName="",
+		     checkedToolTip="", uncheckedToolTip="",
+		     parent=None):
+		ToolButton.__init__(self, uncheckedIconName, "", parent)
+
+		self.__checkedIconName = checkedIconName
+		self.__uncheckedIconName = uncheckedIconName
+		if not self.__uncheckedIconName:
+			self.__uncheckedIconName = self.__checkedIconName
+		self.__checkedToolTip = checkedToolTip
+		self.__uncheckedToolTip = uncheckedToolTip
+		if not self.__uncheckedToolTip:
+			self.__uncheckedToolTip = self.__checkedToolTip
 
 		self.setCheckable(True)
 		self.setChecked(False)
@@ -56,28 +104,22 @@ class RunButton(ToolButton):
 		self.toggled.connect(self.__handleToggle)
 
 	def __handleToggle(self, checked):
-		if checked:
-			self.setIcon(getIcon("stop"))
-			self.setToolTip("Click to stop CPU")
-		else:
-			self.setIcon(getIcon("run"))
-			self.setToolTip("Click to start CPU")
+		self.setToolTip(self.__checkedToolTip if checked else\
+				self.__uncheckedToolTip)
+		self.setIcon(getIcon(self.__checkedIconName if checked else\
+				     self.__uncheckedIconName))
 
 class CpuWidget(QWidget):
 	# Signal: The CPU run-state changed
-	runStateChanged = Signal(int)
+	runStateChanged = Signal(RunState)
 	# Signal: The online-diag state changed
 	onlineDiagChanged = Signal(bool)
+	# Signal: Some configuration value changed
+	configChanged = Signal()
 	# Signal: Have a new instruction dump
 	haveInsnDump = Signal(AwlSimMessage_INSNSTATE)
-
-	EnumGen.start
-	STATE_STOP	= EnumGen.item
-	STATE_INIT	= EnumGen.item
-	STATE_LOAD	= EnumGen.item
-	STATE_RUN	= EnumGen.item
-	STATE_EXCEPTION	= EnumGen.item
-	EnumGen.end
+	# Signal: Have a new ident hashes message
+	haveIdentsMsg = Signal(AwlSimMessage_IDENTS)
 
 	def __init__(self, mainWidget, parent=None):
 		QWidget.__init__(self, parent)
@@ -85,22 +127,46 @@ class CpuWidget(QWidget):
 		self.layout().setContentsMargins(QMargins(7, 0, 0, 0))
 
 		self.mainWidget = mainWidget
-		self.state = self.STATE_STOP
+		self.state = RunState()
+		self.__runStateChangeBlocked = Blocker()
+
+		self.__coreMsgTimer = QTimer(self)
+		self.__coreMsgTimer.setSingleShot(False)
+		self.__coreMsgTimer.timeout.connect(self.__processCoreMessages)
+
+		self.__corePeriodicTimer = QTimer(self)
+		self.__corePeriodicTimer.setSingleShot(False)
+		self.__corePeriodicTimer.timeout.connect(self.__periodicCoreWork)
 
 		client = self.mainWidget.getSimClient()
 		client.haveCpuDump.connect(self.__handleCpuDump)
 		client.haveInsnDump.connect(self.haveInsnDump)
 		client.haveMemoryUpdate.connect(self.__handleMemoryUpdate)
+		client.haveIdentsMsg.connect(self.__handleIdentsMsg)
 
 		toolsLayout = QHBoxLayout()
 
 		group = QGroupBox("CPU", self)
 		group.setLayout(QGridLayout(group))
-		self.runButton = RunButton(group)
-		group.layout().addWidget(self.runButton, 0, 0)
-		self.onlineDiagButton = ToolButton("glasses", "Online diagnosis", group)
-		self.onlineDiagButton.setCheckable(True)
-		group.layout().addWidget(self.onlineDiagButton, 0, 1)
+		self.onlineButton = CheckableToolButton("network", None,
+					"Click to go offline",
+					"Click to go online (Connect to a CPU)",
+					group)
+		group.layout().addWidget(self.onlineButton, 0, 0)
+		self.downloadButton = ToolButton("download",
+					"Download all AWL/STL code to CPU",
+					group)
+		group.layout().addWidget(self.downloadButton, 0, 1)
+		self.runButton = CheckableToolButton("stop", "run",
+					"Click to stop CPU",
+					"Click to start CPU",
+					group)
+		group.layout().addWidget(self.runButton, 0, 2)
+		self.onlineDiagButton = CheckableToolButton("glasses", None,
+					"Online diagnosis",
+					None,
+					group)
+		group.layout().addWidget(self.onlineDiagButton, 0, 3)
 		toolsLayout.addWidget(group)
 
 		group = QGroupBox("Inspection", self)
@@ -126,10 +192,15 @@ class CpuWidget(QWidget):
 		toolsLayout.addStretch()
 		self.layout().addLayout(toolsLayout, 0, 0)
 
-		self.stateWs = StateWorkspace(self)
-		self.stateWs.setScrollBarsEnabled(True)
-		self.layout().addWidget(self.stateWs, 1, 0)
+		self.stateMdi = StateMdiArea(self)
+		self.stateMdi.setViewMode(QMdiArea.SubWindowView)
+		self.stateMdi.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
+		self.stateMdi.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
+		self.layout().addWidget(self.stateMdi, 1, 0)
 
+		self.state.stateChanged.connect(self.runStateChanged)
+		self.onlineButton.toggled.connect(self.__onlineToggled)
+		self.downloadButton.pressed.connect(self.__download)
 		self.runButton.toggled.connect(self.__runStateToggled)
 		self.onlineDiagButton.toggled.connect(self.__updateOnlineViewState)
 		self.newCpuStateButton.released.connect(self.__newWin_CPU)
@@ -144,8 +215,13 @@ class CpuWidget(QWidget):
 		self.__newWin_CPU()
 		self.update()
 
+	def __stateMdiWindowClosed(self, mdiWin):
+		QTimer.singleShot(0, self.__uploadMemReadAreas)
+
 	def __addWindow(self, win):
-		self.stateWs.addWindow(win, Qt.Window)
+		mdiWin = StateMdiSubWindow(win)
+		mdiWin.closed.connect(self.__stateMdiWindowClosed)
+		self.stateMdi.addSubWindow(mdiWin, Qt.Window)
 		win.configChanged.connect(self.__stateWinConfigChanged)
 		win.show()
 		self.update()
@@ -189,7 +265,8 @@ class CpuWidget(QWidget):
 		self.__uploadMemReadAreas()
 
 	def update(self):
-		for win in self.stateWs.windowList():
+		for mdiWin in self.stateMdi.subWindowList():
+			win = mdiWin.widget()
 			win.update()
 
 	# Upload the used memory area descriptors to the core.
@@ -197,7 +274,8 @@ class CpuWidget(QWidget):
 		client = self.mainWidget.getSimClient()
 		wantDump = False
 		memAreas = []
-		for win in self.stateWs.windowList():
+		for mdiWin in self.stateMdi.subWindowList():
+			win = mdiWin.widget()
 			memAreas.extend(win.getMemoryAreas())
 			if isinstance(win, State_CPU):
 				wantDump = True
@@ -207,63 +285,250 @@ class CpuWidget(QWidget):
 						     sync = True)
 			client.setPeriodicDumpInterval(300 if wantDump else 0)
 		except AwlSimError as e:
+			self.state.setState(RunState.STATE_EXCEPTION)
 			MessageBox.handleAwlSimError(self,
-				"Communication error with the simulator core",
-				e)
+				"Error in awlsim core", e)
+			return False
+		except MaintenanceRequest as e:
+			self.__handleMaintenance(e)
+			return False
+		return True
 
 	def __handleCpuDump(self, dumpText):
-		for win in self.stateWs.windowList():
+		for mdiWin in self.stateMdi.subWindowList():
+			win = mdiWin.widget()
 			if isinstance(win, State_CPU):
 				win.setDumpText(dumpText)
 
 	def __handleMemoryUpdate(self, memAreas):
-		for win in self.stateWs.windowList():
+		for mdiWin in self.stateMdi.subWindowList():
+			win = mdiWin.widget()
 			win.setMemories(memAreas)
 
-	def __run(self):
+	def __handleMaintenance(self, maintRequest):
 		client = self.mainWidget.getSimClient()
 
-		self.__setState(self.STATE_INIT)
+		if maintRequest.requestType == MaintenanceRequest.TYPE_SHUTDOWN:
+			res = QMessageBox.question(self,
+				"Shut down application?",
+				"The core server requested an "
+				"application shutdown.\n"
+				"Do you want to close Awlsim GUI?",
+				QMessageBox.Yes | QMessageBox.No,
+				QMessageBox.No)
+			if res == QMessageBox.Yes:
+				print("Shutting down, as requested by server...")
+				client.shutdown()
+				QApplication.exit(0)
+			else:
+				self.stop()
+				self.goOffline()
+		elif maintRequest.requestType == MaintenanceRequest.TYPE_STOP or\
+		     maintRequest.requestType == MaintenanceRequest.TYPE_RTTIMEOUT:
+			self.stop()
+		else:
+			print("Unknown maintenance request %d" %\
+			      maintRequest.requestType)
+			self.stop()
+
+	def __run(self, goOnlineFirst=True, downloadFirstIfSimulator=True):
+		client = self.mainWidget.getSimClient()
+
+		# Make sure the button is pressed.
+		with self.__runStateChangeBlocked:
+			self.runButton.setChecked(True)
+
+		# If requested, go online first.
+		if goOnlineFirst:
+			self.goOnline()
+			if not self.isOnline():
+				self.stop()
+				return
+		assert(self.isOnline())
+
+		# If requested and if in sim (FORK) mode,
+		# download the program first.
+		if downloadFirstIfSimulator and\
+		   client.getMode() == client.MODE_FORK:
+			if not self.__download(noRun=True):
+				self.stop()
+				return
+
+		# Put the CPU and the GUI into RUN state.
+		try:
+			# Put CPU into RUN mode, if it's not already there.
+			client.setRunState(True)
+
+			# Upload the GUI requests.
+			if not self.__uploadMemReadAreas():
+				self.stop()
+				return
+			self.__updateOnlineViewState()
+
+			# Put the GUI into RUN mode.
+			self.state.setState(RunState.STATE_RUN)
+			self.__identsPending = False
+			self.__periodicCoreWork()
+			self.__coreMsgTimer.start(0)
+			self.__corePeriodicTimer.start(1000)
+		except AwlSimError as e:
+			self.state.setState(RunState.STATE_EXCEPTION)
+			MessageBox.handleAwlSimError(self,
+				"Could not start CPU", e)
+			self.stop()
+		except MaintenanceRequest as e:
+			self.__handleMaintenance(e)
+
+	# Periodic timer for core message handling.
+	def __processCoreMessages(self):
+		client = self.mainWidget.getSimClient()
+		try:
+			# Receive messages, until we hit a timeout
+			while client.processMessages(0.1):
+				pass
+		except AwlSimError as e:
+			self.state.setState(RunState.STATE_EXCEPTION)
+			MessageBox.handleAwlSimError(self,
+				"Core server error", e)
+			self.stop()
+		except MaintenanceRequest as e:
+			self.__handleMaintenance(e)
+		except Exception:
+			CALL_NOEX(client.setRunState, False)
+			client.shutdown()
+			handleFatalException(self)
+
+	# Periodic timer for core status work.
+	def __periodicCoreWork(self):
+		client = self.mainWidget.getSimClient()
+
+		if not self.__identsPending:
+			self.__identsPending = True
+			client.requestIdents(reqAwlSources = True,
+					     reqSymTabSources = True)
+
+	def __handleIdentsMsg(self, identsMsg):
+		if self.__identsPending:
+			self.__identsPending = False
+			self.haveIdentsMsg.emit(identsMsg)
+
+	def __stop(self):
+		# Make sure the button is released.
+		with self.__runStateChangeBlocked:
+			self.runButton.setChecked(False)
+
+		self.__coreMsgTimer.stop()
+		self.__corePeriodicTimer.stop()
+		if self.isOnline():
+			client = self.mainWidget.getSimClient()
+			try:
+				client.setRunState(False)
+			except AwlSimError as e:
+				MessageBox.handleAwlSimError(self,
+					"Could not stop CPU", e)
+		self.state.setState(RunState.STATE_ONLINE)
+
+	def stop(self):
+		self.runButton.setChecked(False)
+
+	def run(self):
 		self.runButton.setChecked(True)
 
+	def __goOnline(self):
+		project = self.mainWidget.getProject()
+
+		if LinkConfigDialog.askWhenConnecting():
+			dlg = LinkConfigDialog(project, self)
+			dlg.contentChanged.connect(self.configChanged)
+			if dlg.exec_() != LinkConfigDialog.Accepted:
+				self.onlineButton.setChecked(False)
+				return
+
+		linkConfig = project.getCoreLinkSettings()
+		client = self.mainWidget.getSimClient()
+		try:
+			if linkConfig.getSpawnLocalEn():
+				portRange = linkConfig.getSpawnLocalPortRange()
+				interp = linkConfig.getSpawnLocalInterpreterList()
+				if isWinStandalone:
+					# Run the frozen standalone server process
+					client.setMode_FORK(portRange = portRange,
+						serverExecutable = "awlsim-server-module.exe")
+				else:
+					client.setMode_FORK(portRange = portRange,
+						interpreterList = interp)
+				host = port = None
+			else:
+				host = linkConfig.getConnectHost()
+				port = linkConfig.getConnectPort()
+				timeout = linkConfig.getConnectTimeoutMs() / 1000.0
+				client.setMode_ONLINE(host = host,
+						      port = port,
+						      timeout = timeout)
+
+			self.state.setCoreDetails(spawned = linkConfig.getSpawnLocalEn(),
+						  host = host,
+						  port = port)
+			self.state.setState(RunState.STATE_ONLINE)
+
+			if client.getRunState():
+				# The core is already running.
+				# Set the GUI to run state, too.
+				self.__run(goOnlineFirst = False,
+					   downloadFirstIfSimulator = False)
+		except AwlSimError as e:
+			CALL_NOEX(client.setMode_OFFLINE)
+			MessageBox.handleAwlSimError(self,
+				"Error while trying to connect to CPU", e)
+			self.onlineButton.setChecked(False)
+			return
+		except MaintenanceRequest as e:
+			self.__handleMaintenance(e)
+
+	def __goOffline(self):
+		client = self.mainWidget.getSimClient()
+		try:
+			client.setMode_OFFLINE()
+		except AwlSimError as e:
+			MessageBox.handleAwlSimError(self,
+				"Error while trying to disconnect from CPU", e)
+		# Release the stop-button.
+		# This will _not_ stop the CPU, as we're offline already.
+		self.runButton.setChecked(False)
+		self.state.setState(RunState.STATE_OFFLINE)
+
+	def __onlineToggled(self):
+		if self.isOnline():
+			self.__goOnline()
+		else:
+			self.__goOffline()
+
+	def isOnline(self):
+		return self.onlineButton.isChecked()
+
+	def goOnline(self):
+		self.onlineButton.setChecked(True)
+
+	def goOffline(self):
+		self.onlineButton.setChecked(False)
+
+	def __download(self, noRun=False):
+		# Make sure we are online.
+		self.goOnline()
+		if not self.isOnline():
+			return False
+
+		client = self.mainWidget.getSimClient()
 		project = self.mainWidget.getProject()
 		awlSources = self.mainWidget.projectWidget.getAwlSources()
 		symTabSources = self.mainWidget.projectWidget.getSymTabSources()
 		libSelections = self.mainWidget.projectWidget.getLibSelections()
 		if not all(awlSources) or not all(symTabSources):
-			self.stop()
-			return
+			return False
 
 		try:
-			if self.mainWidget.coreConfigDialog.shouldSpawnServer():
-				firstPort, lastPort = self.mainWidget.coreConfigDialog.getSpawnPortRange()
-				interp = self.mainWidget.coreConfigDialog.getInterpreterList()
-				host = AwlSimServer.DEFAULT_HOST
-				for port in range(firstPort, lastPort + 1):
-					if not AwlSimServer.portIsUnused(host, port):
-						continue
-					# XXX: There is a race-window here. Another process might
-					#      allocate the port that we just checked
-					#      before our server is able to allocate it.
-					if isWinStandalone:
-						# Run the py2exe standalone server process
-						client.spawnServer(serverExecutable = "awlsim-server-module.exe",
-								   listenHost = host,
-								   listenPort = port)
-					else:
-						client.spawnServer(interpreter = interp,
-								   listenHost = host,
-								   listenPort = port)
-					break
-				else:
-					raise AwlSimError("Did not find a free port to run the "
-						"awlsim core server on.\nTried port %d to %d on '%s'." %\
-						(firstPort, lastPort, host))
-			else:
-				host = self.mainWidget.coreConfigDialog.getConnectHost()
-				port = self.mainWidget.coreConfigDialog.getConnectPort()
-			client.connectToServer(host = host,
-					       port = port)
+			self.state.setState(RunState.STATE_LOAD)
+
 			client.setRunState(False)
 			client.reset()
 
@@ -271,10 +536,6 @@ class CpuWidget(QWidget):
 			client.enableOBTempPresets(project.getObTempPresetsEn())
 			client.enableExtendedInsns(project.getExtInsnsEn())
 
-			self.__uploadMemReadAreas()
-			self.__updateOnlineViewState()
-
-			self.__setState(self.STATE_LOAD)
 			client.loadHardwareModule("dummy")
 			for symTabSource in symTabSources:
 				client.loadSymbolTable(symTabSource)
@@ -282,90 +543,45 @@ class CpuWidget(QWidget):
 				client.loadLibraryBlock(libSel)
 			for awlSource in awlSources:
 				client.loadCode(awlSource)
-			client.setRunState(True)
+
+			self.state.setState(RunState.STATE_ONLINE)
 		except AwlParserError as e:
-			self.__setState(self.STATE_EXCEPTION)
+			self.state.setState(RunState.STATE_ONLINE)
+			self.runButton.setChecked(False)
 			MessageBox.handleAwlParserError(self, e)
-			self.stop()
-			client.shutdown()
-			return
+			return False
 		except AwlSimError as e:
-			self.__setState(self.STATE_EXCEPTION)
+			self.state.setState(RunState.STATE_ONLINE)
+			self.runButton.setChecked(False)
 			MessageBox.handleAwlSimError(self,
 				"Error while loading code", e)
-			self.stop()
-			client.shutdown()
-			return
-		except Exception:
-			try:
-				client.setRunState(False)
-			except: pass
-			client.shutdown()
-			handleFatalException(self)
-		self.__setState(self.STATE_RUN)
-		try:
-			# The main loop
-			while self.state == self.STATE_RUN:
-				# Receive messages, until we hit a timeout
-				while client.processMessages(0.1):
-					pass
-				# Process GUI events
-				QApplication.processEvents(QEventLoop.AllEvents)
-		except AwlSimError as e:
-			self.__setState(self.STATE_EXCEPTION)
-			MessageBox.handleAwlSimError(self,
-				"Error while executing code", e)
-			self.stop()
+			return False
 		except MaintenanceRequest as e:
-			if e.requestType == MaintenanceRequest.TYPE_SHUTDOWN:
-				print("Shutting down, as requested...")
-				client.shutdown()
-				QApplication.exit(0)
-				return
-			elif e.requestType == MaintenanceRequest.TYPE_STOP or\
-			     e.requestType == MaintenanceRequest.TYPE_RTTIMEOUT:
-				self.stop()
-			else:
-				print("Unknown maintenance request %d" % e.requestType)
-				self.stop()
+			self.__handleMaintenance(e)
+			return False
 		except Exception:
-			try:
-				client.setRunState(False)
-			except: pass
 			client.shutdown()
 			handleFatalException(self)
-		try:
-			client.setRunState(False)
-		except: pass
-		client.shutdown()
 
-	def stop(self):
-		if self.state == self.STATE_STOP:
-			return
-		self.runButton.setChecked(False)
-		self.__setState(self.STATE_STOP)
+		# If we were RUNning before download, put
+		# the CPU into RUN state again.
+		if self.runButton.isChecked() and not noRun:
+			self.__run(goOnlineFirst = False,
+				   downloadFirstIfSimulator = False)
 
-	def run(self):
-		if self.state != self.STATE_STOP:
-			return
-		self.runButton.setChecked(True)
-
-	def getState(self):
-		return self.state
-
-	def __setState(self, newState):
-		if newState != self.state:
-			self.state = newState
-			self.runStateChanged.emit(self.state)
-			QApplication.processEvents(QEventLoop.AllEvents, 1000)
+		return True
 
 	def __runStateToggled(self):
+		if self.__runStateChangeBlocked:
+			return
 		if self.runButton.isChecked():
-			if self.state == self.STATE_STOP:
-				self.__run()
+			self.__run()
 		else:
-			if self.state != self.STATE_STOP:
-				self.stop()
+			self.__stop()
+
+	def handleDirtyChange(self, dirty):
+		if dirty:
+			self.onlineDiagButton.setChecked(False)
 
 	def __updateOnlineViewState(self):
 		onlineDiagEn = self.onlineDiagButton.isChecked()
@@ -386,3 +602,6 @@ class CpuWidget(QWidget):
 			MessageBox.handleAwlSimError(self,
 				"Failed to setup instruction dumping", e)
 			return
+		except MaintenanceRequest as e:
+			self.__handleMaintenance(e)
+			return
