@@ -338,6 +338,10 @@ class EditWidget(SourceCodeEdit):
 	codeChanged = Signal()
 	# Signal: Emitted, if the visible source lines changed (e.g. scrolled)
 	visibleRangeChanged = Signal()
+	# Signal: editor<->Cpu source code match changed.
+	# Parameter 0: This EditWidget
+	# Parameter 1: Bool: True -> Editor code does match CPU code.
+	cpuCodeMatchChanged = Signal(SourceCodeEdit, bool)
 	# Signal: Keyboard focus in/out event.
 	focusChanged = Signal(bool)
 
@@ -368,8 +372,9 @@ class EditWidget(SourceCodeEdit):
 
 		self.__source = AwlSource(name = "Unnamed source")
 		self.__needSourceUpdate = True
+		self.__sourceMatchesCpuSource = False
 
-		self.__runStateCopy = CpuWidget.STATE_STOP
+		self.__runState = RunState()
 		self.__nextHdrUpdate = 0
 		self.__hdrAniStat = 0
 
@@ -382,7 +387,7 @@ class EditWidget(SourceCodeEdit):
 				     CpuStatsEntry.SHOW_ACCU2)
 		self.resetCpuStats(True)
 
-		self.__textChangeBlocked = 0
+		self.__textChangeBlocked = Blocker()
 		self.textChanged.connect(self.__textChanged)
 
 		self.blockCountChanged.connect(self.__updateMargins)
@@ -409,8 +414,7 @@ class EditWidget(SourceCodeEdit):
 		garbageCollector.collect()
 
 	def setSource(self, source):
-		self.__textChangeBlocked += 1
-		try:
+		with self.__textChangeBlocked:
 			self.__source = source.dup()
 			self.__source.sourceBytes = b""
 			sourceBytes = source.sourceBytes
@@ -425,9 +429,7 @@ class EditWidget(SourceCodeEdit):
 								errors="ignore")
 			self.setPlainText(sourceText)
 			self.resetCpuStats()
-		finally:
-			self.__textChangeBlocked -= 1
-			self.__needSourceUpdate = True
+		self.__needSourceUpdate = True
 
 	def __updateSource(self):
 		sourceText = self.toPlainText()
@@ -465,10 +467,13 @@ class EditWidget(SourceCodeEdit):
 		self.__updateFont(font)
 
 	def runStateChanged(self, newState):
-		self.__runStateCopy = newState
-		if newState == CpuWidget.STATE_INIT:
+		self.__runState = newState
+		if newState.state == RunState.STATE_LOAD:
 			self.resetCpuStats()
-		if newState == CpuWidget.STATE_RUN:
+			self.__setSourceMatchesCpuSource(True)
+		if newState.state == RunState.STATE_RUN:
+			self.__setSourceMatchesCpuSource(
+				self.__sourceMatchesCpuSource, force = True)
 			self.__aniTimer.start(200)
 		else:
 			self.__aniTimer.stop()
@@ -584,7 +589,7 @@ class EditWidget(SourceCodeEdit):
 			self.__cpuStatsStamp += 1
 
 	def __pruneInvisibleCpuStats(self):
-		if self.__runStateCopy == CpuWidget.STATE_STOP:
+		if self.__runState.state == RunState.STATE_OFFLINE:
 			return
 		firstLine, lastLine = self.getVisibleLineRange()
 		for line, stats in self.__lineCpuStats.items():
@@ -597,6 +602,22 @@ class EditWidget(SourceCodeEdit):
 				    len(self.__aniChars)
 		self.headerWidget.update()
 
+	def __setSourceMatchesCpuSource(self, sourceIsOnCpu, force=False):
+		if sourceIsOnCpu != self.__sourceMatchesCpuSource or force:
+			self.__sourceMatchesCpuSource = sourceIsOnCpu
+			self.headerWidget.update()
+			self.cpuCodeMatchChanged.emit(self, sourceIsOnCpu)
+
+	def handleIdentsMsg(self, identsMsg):
+		if self.__runState.state == RunState.STATE_RUN:
+			cpuHashes = [ s.identHash for s in identsMsg.awlSources ]
+			self.__setSourceMatchesCpuSource(
+				self.getSource().identHash in cpuHashes)
+		else:
+			# The CPU is not in RUN state.
+			# We don't make a big deal out of code mismatch.
+			self.__setSourceMatchesCpuSource(True)
+
 	def __forwardWheelEvent(self, ev):
 		self.wheelEvent(ev)
 
@@ -604,7 +625,6 @@ class EditWidget(SourceCodeEdit):
 		digi, bcnt = 1, self.blockCount()
 		while bcnt > 9:
 			digi, bcnt = digi + 1, bcnt // 10
-		digi += 1 # colon
 		metr = self.lineNumWidget.fontMetrics()
 		return 5 + 5 + metr.width("_" * digi)
 
@@ -696,21 +716,47 @@ class EditWidget(SourceCodeEdit):
 		self.focusChanged.emit(False)
 
 	__runStateToText = {
-		CpuWidget.STATE_STOP		: "-- CPU STOPPED --",
-		CpuWidget.STATE_INIT		: "Initializing simulator...",
-		CpuWidget.STATE_LOAD		: "Loading code...",
-		CpuWidget.STATE_EXCEPTION	: "ERROR. CPU halted.",
+		RunState.STATE_OFFLINE		: "OFFLINE",
+		RunState.STATE_ONLINE		: "Online (CPU stopped)",
+		RunState.STATE_LOAD		: "DOWNLOADING program. Please wait.",
+		RunState.STATE_EXCEPTION	: "ERROR. CPU halted.",
 	}
 
 	def __repaintHeaderWidget(self, ev):
 		p = self.headerWidget.getPainter()
 		p.fillRect(ev.rect(), Qt.lightGray)
 
-		if self.__runStateCopy == CpuWidget.STATE_RUN:
-			runText = self.__aniChars[self.__hdrAniStat]
+		if self.__cpuStatsEnabled:
+			# Map the CPU-stats start point to header widget
+			# coordinates.
+			cpuStatsPt = self.cpuStatsWidget.mapToGlobal(QPoint(0, 0))
+			cpuStatsPt = self.headerWidget.mapFromGlobal(cpuStatsPt)
+
+			textMaxPixels = cpuStatsPt.x()
 		else:
-			runText = self.__runStateToText[self.__runStateCopy]
+			textMaxPixels = self.headerWidget.width()
+
+		if self.__runState.spawned:
+			runText = "[SIM]: "
+		else:
+			runText = "[%s:%d]: " %(\
+				self.__runState.host,
+				self.__runState.port)
+		if self.__runState.state == RunState.STATE_RUN:
+			runText += self.__aniChars[self.__hdrAniStat]
+			if not self.__sourceMatchesCpuSource:
+				runText += " [NOT DOWNLOADED to CPU]"
+		else:
+			runText += self.__runStateToText[self.__runState.state]
+
+		# Limit the text length to the available space.
 		metr = self.headerWidget.fontMetrics()
+		maxNrChars = textMaxPixels // metr.width('_')
+		if len(runText) > maxNrChars:
+			runText = runText[ : max(0, maxNrChars - 3)]
+			runText += "..."
+			runText = runText[ : maxNrChars]
+
 		p.drawText(5, 5,
 			   metr.width(runText.replace(" ", "_")),
 			   self.headerWidget.height(),
@@ -718,12 +764,8 @@ class EditWidget(SourceCodeEdit):
 			   runText)
 
 		if self.__cpuStatsEnabled:
-			# Map the starting point
-			pt = self.cpuStatsWidget.mapToGlobal(QPoint(0, 0))
-			pt = self.headerWidget.mapFromGlobal(pt)
-
-			p.drawText(pt.x() + 5, 5,
-				   self.headerWidget.width() - pt.x() - 5,
+			p.drawText(cpuStatsPt.x() + 5, 5,
+				   self.headerWidget.width() - cpuStatsPt.x() - 5,
 				   self.headerWidget.height(),
 				   Qt.AlignLeft,
 				   self.cpuStatsWidget.getBanner(self.__cpuStatsMask))
@@ -742,7 +784,7 @@ class EditWidget(SourceCodeEdit):
 				   self.lineNumWidget.width(),
 				   self.__charHeight,
 				   Qt.AlignRight,
-				   "%d:" % lineNr)
+				   str(lineNr))
 
 	def __repaintCpuStatsWidget(self, ev):
 		p = self.cpuStatsWidget.getPainter()
@@ -770,6 +812,11 @@ class EditWidget(SourceCodeEdit):
 		self.__needSourceUpdate = True
 		self.codeChanged.emit()
 		self.resetCpuStats()
+		if self.__runState.state != RunState.STATE_RUN:
+			# The CPU is not in RUN state.
+			# We don't make a big deal out of code mismatch, even
+			# if the code most likely _does_ mismatch after this edit.
+			self.__setSourceMatchesCpuSource(True)
 
 	# Validation callback. Overridden subclass method.
 	def validateText(self, text, currentLineNr):
