@@ -30,15 +30,17 @@ from awlsim.gui.editwidget import *
 from awlsim.gui.projectwidget import *
 from awlsim.gui.guiconfig import *
 from awlsim.gui.cpuconfig import *
-from awlsim.gui.coreconfig import *
+from awlsim.gui.linkconfig import *
 from awlsim.gui.icons import *
 
 
 class MainWidget(QWidget):
+	# Signal: Project loaded
+	projectLoaded = Signal(Project)
 	# Signal: Dirty-status changed
 	dirtyChanged = Signal(bool)
 	# Signal: CPU run state changed
-	runStateChanged = Signal(int)
+	runStateChanged = Signal(RunState)
 	# Signal: Source text focus changed
 	textFocusChanged = Signal(bool)
 	# Signal: Selected project resource changed
@@ -56,8 +58,6 @@ class MainWidget(QWidget):
 
 		self.simClient = GuiAwlSimClient()
 
-		self.coreConfigDialog = CoreConfigDialog(self, self.simClient)
-
 		self.splitter = QSplitter(Qt.Horizontal)
 		self.layout().addWidget(self.splitter, 0, 0)
 
@@ -79,10 +79,13 @@ class MainWidget(QWidget):
 		self.projectWidget.undoAvailableChanged.connect(self.undoAvailableChanged)
 		self.projectWidget.redoAvailableChanged.connect(self.redoAvailableChanged)
 		self.projectWidget.copyAvailableChanged.connect(self.copyAvailableChanged)
-		self.cpuWidget.runStateChanged.connect(self.__runStateChanged)
+		self.cpuWidget.runStateChanged.connect(self.runStateChanged)
 		self.cpuWidget.onlineDiagChanged.connect(self.projectWidget.handleOnlineDiagChange)
 		self.cpuWidget.haveInsnDump.connect(self.projectWidget.handleInsnDump)
+		self.cpuWidget.haveIdentsMsg.connect(self.projectWidget.handleIdentsMsg)
+		self.cpuWidget.configChanged.connect(self.__somethingChanged)
 		self.runStateChanged.connect(self.projectWidget.updateRunState)
+		self.dirtyChanged.connect(self.cpuWidget.handleDirtyChange)
 
 	def isDirty(self):
 		return self.dirty
@@ -92,12 +95,12 @@ class MainWidget(QWidget):
 			self.dirty = dirty
 			self.dirtyChanged.emit(self.dirty)
 
+	def __somethingChanged(self):
+		self.setDirty(True)
+
 	def getFilename(self):
 		return self.filename
 
-	def __runStateChanged(self, newState):
-		self.runStateChanged.emit(newState)
-
 	def getSimClient(self):
 		return self.simClient
 
@@ -107,10 +110,6 @@ class MainWidget(QWidget):
 	def getProject(self):
 		return self.projectWidget.getProject()
 
-	def __somethingChanged(self):
-		self.cpuWidget.stop()
-		self.setDirty(True)
-
 	def newFile(self, filename=None):
 		if isWinStandalone:
 			executableName = "awlsim-gui.exe"
@@ -156,6 +155,7 @@ class MainWidget(QWidget):
 				return
 			else:
 				assert(0)
+		self.cpuWidget.goOffline()
 		try:
 			res = self.projectWidget.loadProjectFile(filename)
 			if not res:
@@ -167,6 +167,7 @@ class MainWidget(QWidget):
 		self.filename = filename
 		self.setDirty(dirty = not bool(self.getProject().getProjectFile()),
 			      force = True)
+		self.projectLoaded.emit(self.getProject())
 		return True
 
 	def load(self):
@@ -218,6 +219,12 @@ class MainWidget(QWidget):
 			self.projectWidget.setSettings(project.getGuiSettings())
 			self.__somethingChanged()
 
+	def linkConfig(self):
+		project = self.getProject()
+		dlg = LinkConfigDialog(project, self)
+		dlg.contentChanged.connect(self.__somethingChanged)
+		dlg.exec_()
+
 	def cpuConfig(self):
 		project = self.getProject()
 		dlg = CpuConfigDialog(self)
@@ -226,9 +233,6 @@ class MainWidget(QWidget):
 			dlg.saveToProject(project)
 			self.__somethingChanged()
 
-	def coreConfig(self):
-		self.coreConfigDialog.exec_()
-
 	def insertOB(self):
 		self.projectWidget.insertOB()
 
@@ -278,6 +282,12 @@ class MainWindow(QMainWindow):
 		  initialAwlSource = None):
 		if not qApplication:
 			qApplication = QApplication(sys.argv)
+		# Set basic qapp-details.
+		# This is important for QSettings.
+		qApplication.setOrganizationName("awlsim")
+		qApplication.setApplicationName("Awlsim GUI")
+		qApplication.setApplicationVersion(VERSION_STRING)
+
 		mainwnd = cls(initialAwlSource)
 		mainwnd.show()
 		return mainwnd
@@ -322,7 +332,7 @@ class MainWindow(QMainWindow):
 		self.menuBar().addMenu(menu)
 
 		menu = QMenu("&Settings", self)
-		menu.addAction(getIcon("network"), "&Server connection...", self.coreConfig)
+		menu.addAction(getIcon("network"), "&Server connection...", self.linkConfig)
 		menu.addAction(getIcon("prefs"), "&CPU config...", self.cpuConfig)
 		menu.addAction(getIcon("prefs"), "&User interface...", self.guiConfig)
 		self.menuBar().addMenu(menu)
@@ -349,14 +359,14 @@ class MainWindow(QMainWindow):
 		self.addToolBar(self.tb)
 
 		self.__dirtyChanged(False)
-		self.__selResourceChanged(None)
+		self.__selResourceChanged(ProjectWidget.RES_SOURCES)
 		self.__textFocusChanged(False)
 		self.__undoAvailableChanged(False)
 		self.__redoAvailableChanged(False)
 		self.__copyAvailableChanged(False)
 
+		self.centralWidget().projectLoaded.connect(self.__handleProjectLoaded)
 		self.centralWidget().dirtyChanged.connect(self.__dirtyChanged)
-		self.centralWidget().runStateChanged.connect(self.__runStateChanged)
 		self.centralWidget().textFocusChanged.connect(self.__textFocusChanged)
 		self.centralWidget().selResourceChanged.connect(self.__selResourceChanged)
 		self.centralWidget().undoAvailableChanged.connect(self.__undoAvailableChanged)
@@ -396,16 +406,6 @@ class MainWindow(QMainWindow):
 	def getSimClient(self):
 		return self.centralWidget().getSimClient()
 
-	def cpuRun(self):
-		self.centralWidget().getCpuWidget().run()
-
-	def cpuStop(self):
-		self.centralWidget().getCpuWidget().stop()
-
-	def __runStateChanged(self, newState):
-		self.menuBar().setEnabled(newState == CpuWidget.STATE_STOP)
-		self.tb.setEnabled(newState == CpuWidget.STATE_STOP)
-
 	def __dirtyChanged(self, isDirty):
 		self.saveAct.setEnabled(isDirty)
 		self.tbSaveAct.setEnabled(isDirty)
@@ -417,9 +417,11 @@ class MainWindow(QMainWindow):
 				postfix += "*"
 		else:
 			postfix = ""
-		self.setWindowTitle("AWL/STL soft-PLC v%d.%d%s" %\
-				    (VERSION_MAJOR, VERSION_MINOR,
-				     postfix))
+		self.setWindowTitle("AWL/STL soft-PLC v%s%s" %\
+				    (VERSION_STRING, postfix))
+
+	def __handleProjectLoaded(self, project):
+		self.__updateLibActions()
 
 	def __updateLibActions(self):
 		# Enable/disable the library toolbar button.
@@ -481,18 +483,6 @@ class MainWindow(QMainWindow):
 		self.__updateClipboardActions()
 
 	def closeEvent(self, ev):
-		cpuWidget = self.centralWidget().getCpuWidget()
-		if cpuWidget.getState() != CpuWidget.STATE_STOP:
-			res = QMessageBox.question(self,
-				"CPU is in RUN state",
-				"CPU is in RUN state.\n"
-				"STOP CPU and quit application?",
-				QMessageBox.Yes | QMessageBox.No,
-				QMessageBox.Yes)
-			if res != QMessageBox.Yes:
-				ev.ignore()
-				return
-		self.centralWidget().getCpuWidget().stop()
 		if self.centralWidget().isDirty():
 			res = QMessageBox.question(self,
 				"Unsaved AWL/STL code",
@@ -507,6 +497,7 @@ class MainWindow(QMainWindow):
 			elif res == QMessageBox.Cancel:
 				ev.ignore()
 				return
+		self.centralWidget().getSimClient().shutdown()
 		ev.accept()
 		QMainWindow.closeEvent(self, ev)
 
@@ -515,7 +506,7 @@ class MainWindow(QMainWindow):
 
 	def about(self):
 		QMessageBox.about(self, "About AWL/STL soft-PLC",
-			"Awlsim soft-PLC version %d.%d\n"
+			"Awlsim soft-PLC version %s\n"
 			"\n"
 			"Copyright 2012-2014 Michael Büsch <m@bues.ch>\n"
 			"\n"
@@ -535,8 +526,7 @@ class MainWindow(QMainWindow):
 			"You should have received a copy of the GNU General Public License along "
 			"with this program; if not, write to the Free Software Foundation, Inc., "
 			"51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA." %\
-			(VERSION_MAJOR, VERSION_MINOR,
-			 AWLSIM_HOME_URL))
+			(VERSION_STRING, AWLSIM_HOME_URL))
 
 	def new(self):
 		self.centralWidget().newFile()
@@ -553,12 +543,12 @@ class MainWindow(QMainWindow):
 	def guiConfig(self):
 		self.centralWidget().guiConfig()
 
+	def linkConfig(self):
+		self.centralWidget().linkConfig()
+
 	def cpuConfig(self):
 		self.centralWidget().cpuConfig()
 
-	def coreConfig(self):
-		self.centralWidget().coreConfig()
-
 	def openLibrary(self):
 		self.centralWidget().openLibrary()
 
