@@ -50,6 +50,9 @@ import os
 import traceback
 
 
+AWLSIM_HOME_URL = "http://bues.ch/h/awlsim"
+
+
 # Convert an integer to a dual-string
 def intToDualString(value, bitWidth):
 	string = []
@@ -59,13 +62,14 @@ def intToDualString(value, bitWidth):
 			string.append('_')
 	return ''.join(string)
 
-# Set the specified QFont to fixed-font
-def setFixedFontParams(font, pointSize=10):
-	font.setFamily("courier")
-	font.setPointSize(pointSize)
-	font.setKerning(False)
-	font.setFixedPitch(True)
-	font.setStyleHint(QFont.TypeWriter, QFont.PreferBitmap)
+# Get the default fixed font
+def getDefaultFixedFont():
+	font = QFont()
+	font.setStyleHint(QFont.Courier)
+	font.setFamily("Courier")
+	font.setPointSize(11)
+	font.setWeight(QFont.Normal)
+	return font
 
 def handleFatalException(parentWidget=None):
 	text = str(traceback.format_exc())
